% function    outt    =   Complexity_Analysis(sett,fsens,out)
%
%   Description:
%       Main file for the complexity analysis with the configuration given 
%       by the input variables. Analysis of the complexity for different 
%       values of signal bandwidth, chirp period,constellation size and 
%       number of visible satellites (when fsens = 1) 
%
%   Inputs:
%       sett    =   Structure with setting parameters (see setParams.m)
%
%       fsens   =   Sensitivity analysis flag
%
%       out     =   Structure with output results. Useful when fsens = 1 to
%                   get the results from the sensitivity analysis and link
%                   it with the complexity analysis
%
%   Outputs:    
%       outt.C  =   (NBxNTcxNmxNvis) matrix with the complexity value for
%                   the different NB bandwidth values, NTc chirp period
%                   values, Nm constellation sizes and Nvis number of
%                   visible satellites.
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $8-jul-2021$ $12-jun-2021$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    outt    =   Complexity_Analysis(sett,fsens,out)
%
    NB      =   sett.samp.NB;
    Nm      =   length(sett.M);
    Nvis    =   length(sett.Mvis);
    if fsens
        opt     =   sett.sens.opt;
        flg_nc  =   sett.sens.flg_nc;
        if( opt == 2 )
            Tc  =   out.sens.Tcopt.Tc;
            NTc     =   size(Tc,2);
        end
        C               =   zeros(NB,NTc,Nm,Nvis);
    else
        NTc     =   sett.samp.NTc;
        if( length(sett.samp.Nnc) > 1 )
            flg_nc  =   1;
            NTc     =   1;
            Tcoh    =   (sett.sig.Tc(end) + sett.sig.Tc(1))/2;
            Nnc     =   sett.samp.Nnc;
            C       =   zeros(NB,length(Nnc),Nm,Nvis);
        else
            flg_nc  =   0;
            C               =   zeros(NB,NTc,Nm,Nvis);
        end        
    end
    %    
    for ii = 1:NB
        B           =   sett.sig.B(ii);
        for jj = 1:NTc
            if flg_nc
                if fsens
                    Tcoh    =   sett.sig.Tc;
                    Nnc     =   reshape(Tc(ii,jj,:,:)/Tcoh,Nm,Nvis);
                end
            else
                if fsens
                    switch opt
                        case 1
                            
                        case 2
                            Tcoh    =   reshape(Tc(ii,jj,:,:),Nm,Nvis);
                    end
                else
                    Tcoh    =   sett.sig.Tc(jj);
                end
            end
            for k1 = 1:Nm
                Nsat        =   sett.M(k1);
                for k2 = 1:Nvis
                    if fsens
                        alpha   =   2^nextpow2(B*Tcoh(k1,k2)/2);
                    else
                        alpha   =   2^nextpow2(B*Tcoh/2);
                    end
                    Ct          =   complexityfnct(alpha,Nsat);
                    if flg_nc
                        if fsens
                            C(ii,jj,k1,k2)  =   Nnc(k1,k2)*Ct.Cmds;
                        else
                            C(ii,:,k1,k2)  =   Nnc*Ct.Cmds;
                        end
                    else
                        C(ii,jj,k1,k2)  =   Ct.Cmds;
                    end
                end
            end
        end
    end
    %
    outt        =   C;
                        
                    








%
end