% function    MUI     =   MUI_Analysis(sett,flg)
%
%   Description:
%       Main file for the Multi-User Interference (MUI) analysis with the 
%       configuration given by the input variables sett and flg. Analysis
%       of the MUI for different values of signal bandwidth, chirp period,
%       constellation size and number of visible satellites. 3 values of
%       MUI are provided, based on the maximum, minimum and average values
%       obtained among different sets of visible satellites
%
%   Inputs:
%       sett    =   Structure with setting parameters (see setParams.m)
%
%       flg     =   Structure with flag options (see setParams.m)
%
%   Outputs:    
%       MUI     =   Structure with the results of the MUI analysis
%
%       MUI.min =   Minimum MUI value
%       MUI.max =   Maximum MUI value
%       MUI.avg =   Average MUI value
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $8-jul-2021$ $12-jun-2021$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    MUI     =   MUI_Analysis(sett,flg)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    %
    fsens       =   flg.fsens;          %   Sensitivity analysis flag to link MUI with sensitivity analysis [1] or not [0]
    M           =   sett.M;             %   Constellation size
    Mvis        =   sett.Mvis;          %   Number of visible satellites
    NB          =   sett.samp.NB;       %   Number of bandwidth values to be analyzed
    NTc         =   sett.samp.NTc;      %   Number of chirp period values to be analyzed       
    B           =   sett.sig.B;         %   Bandwidth/s to be analyzed
    Tc          =   sett.sig.Tc;        %   Chirp period/s to be analyzed
    %
    %-  Specific variables initialization
    %
    Nm          =   length(M);
    Nvis        =   length(Mvis);
    tmp         =   sett;
    tmp.flg     =   0;
    %
%----------------------------- RUN MUI ANALYSIS ENGINE -------------------%    
    %
    MUIm                =   zeros(NTc,NB,Nm,Nvis);
    MUIma               =   zeros(NTc,NB,Nm,Nvis);
    MUIa                =   zeros(NTc,NB,Nm,Nvis);
    perc    = 1;
    hwb     =   waitbar(0,'Multi-User Interference Analysis ...');
    for ii = 1:NTc
        %-                              Loop NTc
        %
        tmp.samp.Nnc    =   1;
        if ~fsens
            tmp.sig.Tc      =   Tc(ii);
            disp(['* Tc = ' num2str(Tc(ii)*1e3) ' ms']);
        end
        %
        for jj = 1:NB
            %-                          Loop NB
            %
            tmp.sig.B   =   B(jj);
            disp(['* Bandwidth = ' num2str(B(jj)/1e6) ' MHz']);
            %
            for k1 = 1:Nm
                %-                      Loop Nm
                %
                tmp.M   =   M(k1);
                disp(['* M = ' num2str(M(k1)) ' sats.']);
                %
                %--     Generate signal for given Tc,B and Nsat
                %
                if ~fsens
                    [nset,~]    =   initParams(tmp,flg);
                    nset.flg    =   1;
                    out.BOK     =   genBOKchirp_MultiUser(nset,flg.Muser);
                    nset.flg    =   0;
                end
                %
                for k2 = 1:Nvis
                    %-                  Loop Nvis
                    %
                    nset.Mvis           =   Mvis(k2);
                    disp(['* Mvis = ' num2str(Mvis(k2)) ' sats.']);
                    diro    =   [pwd filesep 'MultiUser' filesep 'Results' filesep];
                    dirr    =   [diro 'B' num2str(B(jj)/1e6) 'MHz'];
                    if( exist(dirr,'dir') == 0 )
                        mkdir(dirr);
                    end
                    str     =   ['B' num2str(B(jj)/1e6) 'MHz_Tc' ...
                                 num2str(Tc(jj,ii,k1,k2)*1e3) 'ms_M' num2str(M(k1)) '_Mv' ...
                                 num2str(Mvis(k2))];


                    strr    =   [dirr filesep 'out_' str '.mat'];
                    %
                    %--     Link with sensitivity
                    %
                    if fsens
                        %---    Get Tc and generate corresponding signal
                        %       with B and Nsat
                        %
                        tmp.sig.Tc      =   Tc(jj,ii,k1,k2);
                        disp(['* Tc = ' num2str(Tc(jj,ii,k1,k2)*1e3) ' ms']);
                        %--     Check if results for given configuration already
                        %       exist (e.g., previously generated, externaly
                        %       generated, ...)
                        %
                        if( exist(strr,'file') == 0 ) %->   Generate results
                            %
                            [nset,~]        =   initParams(tmp,flg);
                            nset.flg        =   1;
                            out.BOK         =   genBOKchirp_MultiUser(nset,...
                                                                    flg.Muser);
                            nset.flg        =   0;
                            %
                        end
                    end
                    %
                    
                    %--     Check if results for given configuration already
                    %       exist (e.g., previously generated, externaly
                    %       generated, ...)
                    %
                    if( exist(strr,'file') == 2 ) %->   Load results
                        %
                        A                   =   load(strr);
                        MUIm(ii,jj,k1,k2)   =   A.MUI.min;
                        MUIma(ii,jj,k1,k2)  =   A.MUI.max;
                        MUIa(ii,jj,k1,k2)   =   A.MUI.avg; 
                        %
                    else
                        %
                        Ck                  =   out.BOK.sig.Ck;                 %   Get signal for all users
                        %--     Run MUI analysis
                        %
                        [tt,MUIt]           =   getMUI(Ck,Mvis(k2));
                        MUIm(ii,jj,k1,k2)   =   min(tt.min);
                        MUIma(ii,jj,k1,k2)  =   max(tt.max);
                        MUIa(ii,jj,k1,k2)   =   mean(tt.avg); 
                        %
                        %--     Save results
                        %
                        MUI.min        =   min(tt.min);
                        MUI.max        =   max(tt.max);
                        MUI.avg        =   mean(tt.avg);
                        save(strr,'MUI');

                        %
                    end
                    %
                    %--     Plot intermediate results
                    %
                    if flg.plot.fmui
                        if flg.plot.mui.users
                            figure;plot(10*log10(MUIt),'--','LineWidth',2);
                            hold on;plot(10*log10(tt.max),'--','LineWidth',2);
                            plot(10*log10(tt.avg),'LineWidth',2);
                            plot(10*log10(tt.min),'--','LineWidth',2);
                            xlabel('User id.');
                            ylabel('MUI (dB)');
                            title(['Tc = ' num2str(Tc(ii)*1e3) ' ms, B = ' ...
                                   num2str(B(jj)/1e6) ' MHz, Mvis = ' num2str(Mvis(k2))]);
                            legend('Consecutive','Max.','Avg.','Min.');
                            grid;
                            set(gca,'fontsize',12);
                        end
                    end
                    %
                end
                %
            end            
            %
        end        
        %
        waitbar(perc/NTc,hwb);
        perc    =   perc + 1;
    end
    close(hwb);
    %
%----------------------------- SAVE MUI STRUCTURE ------------------------%
    MUI.min     =   MUIm;
    MUI.max     =   MUIma;
    MUI.avg     =   MUIa;
    %














%
end