% function    [MUI,MUIt]  =   getMUI(Ck,Nvis)
%
%   Description:
%       Get the MUI for the input signal in Ck and Nvis visible satellites.
%       The result for the maximum, minimum and average over 100 different
%       random sets of visible satellites is provided.
%
%   Inputs:
%       Ck      =   (MxN) matrix with the I/Q samples of the BOK-chirp
%                   signal for the N epochs and M satellites in the
%                   constellation
%       Nvis    =   Number of visible satellites to analyze the MUI
%
%   Outputs:   
%       MUI     =   Structure with the minimum, maximum and average MUI
%                   values
%           .min
%           .max
%           .avg
%       MUIt    =   MUI value when considering Nvis consecutive satellites
%
%--------------------------------------------------------------------------
%   @version    $3.0$
%   @date       $8-jul-2021$ $12-jun-2021$ $8-may-2020$
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    [MUI,MUIt]  =   getMUI(Ck,Nvis)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    R           =   1e2;              %   Number of different random sets of visible satellites
    M           =   size(Ck,1);
    N           =   size(Ck,2);
    %
    Ps          =   Nvis*2;
    MUI_min     =   zeros(M,1);
    MUI_max     =   zeros(M,1);
    MUI_avg     =   zeros(M,1);
    MUIt        =   zeros(M,1);
    %
    idxt    =   2:Nvis;
    perc    =   1;
    %
    for ii = 1:M
        %
        if( (ii/M)*100 >= 10*perc )
            disp(['> ' num2str(10*perc) ' % ...']);
            perc    =   perc + 1;
        elseif( ii == M )
            disp('> Done!');
            disp('% ...');
        end
        %
        Pr              =   zeros(R,1);
        if( ii <= Nvis )
            idxc        =   [ii (M - Nvis + 2):M];
        else
            idxc        =   [ii idxt];
        end
        Ct          =   sum(Ck(idxc,:));
        Prc         =   (Ct*Ct')/N;
        MUIt(ii)    =   Prc/Ps;
        for kk = 1:R
            %
            if( Nvis < M )
                idx         =   randperm(M,Nvis-1).';
                while( ~isempty(find(idx == ii,1)) || ~isempty(find(idx == (M-ii+2),1)) )
                    idx     =   randperm(M,Nvis-1).';
                end
                idsat       =   [ii;idx];
            else
                idsat       =   1:M;
            end
            C           =   sum(Ck(idsat,:));    
             
            Pr(kk)      =   (C*C')/N;
            
            %
        end
        MUI_min(ii)     =   min(Pr/Ps);
        MUI_max(ii)     =   max(Pr/Ps);
        MUI_avg(ii)     =   mean(Pr/Ps);
        %
    end
    %
    MUI.min     =   MUI_min;
    MUI.max     =   MUI_max;
    MUI.avg     =   MUI_avg;
    %
    
    %

end

