% function    outt    =   RMSE_Analysis(sett,flgt)
%
%   Description:
%       Main file for the accuracy analysis with the configuration given by 
%       the input variables sett and flg. Analysis of the RMSE for different 
%       values of signal bandwidth, chirp period, constellation size and 
%       number of visible satellites. When the MUI analysis is also
%       selected, 3 values of RMS are provided, based on the maximum, 
%       minimum and average values of the MUI. These values are based on
%       the CRLB of a BOK-chirp signal and considering the degradation due
%       to the MUI. Numerical simulations based on Monte-Carlo realizations
%       is also possible
%
%   Inputs:
%       sett    =   Structure with setting parameters (see setParams.m)
%
%       flg     =   Structure with flag options (see setParams.m)
%
%   Outputs:    
%       outt    =   Structure with the results of the MUI analysis
%
%       outt.tau        =   (NTcxNBxNcxNmxNvis) matrix with the numerical
%                           simulation of the time-delay RMSE for the given 
%                           NTc chirp period values, NB bandwidths, Nc
%                           CN0s, Nm number of constellation sizes and Nvis
%                           values of visible satellites (result for 1
%                           satellite)
%       outt.fD         =   (NTcxNBxNcxNmxNvis) matrix with the numerical
%                           simulation of the Doppler RMSE for the given 
%                           NTc chirp period values, NB bandwidths, Nc
%                           CN0s, Nm number of constellation sizes and Nvis
%                           values of visible satellites
%       outt.CRLB       =   (NTc,NB,Nc) matrix with the values of the 
%                           (time-delay)CRLB of a BOK-chirp for the 
%                           different NTc chirp period values, NB 
%                           bandwidths and Nc CN0 values
%       outt.CRLBf      =   (NTc,NB,Nc) matrix with the values of the 
%                           (Doppler)CRLB of a BOK-chirp for the 
%                           different NTc chirp period values, NB 
%                           bandwidths and Nc CN0 values
% (fMUI = 1)
%       outt.CRLBavg    =   (NTc,NB,Nc) matrix with the values of the 
%                           (time-delay)CRLB of a multi-satellite BOK-chirp  
%                           for the different NTc chirp period values, NB 
%                           bandwidths and Nc CN0 values. The average value
%                           considering the average MUI is provided.
%              
%       outt.CRLBma     =   (NTc,NB,Nc) matrix with the values of the 
%                           (time-delay)CRLB of a multi-satellite BOK-chirp  
%                           for the different NTc chirp period values, NB 
%                           bandwidths and Nc CN0 values. The maximum value
%                           considering the maximum MUI is provided.
%       outt.CRLBmi     =   (NTc,NB,Nc) matrix with the values of the 
%                           (time-delay)CRLB of a multi-satellite BOK-chirp  
%                           for the different NTc chirp period values, NB 
%                           bandwidths and Nc CN0 values. The minimum value
%                           considering the minimum MUI is provided.
% (const = 1)
%       outt.MU         =   (NTcxNBxNcxNmxNvis) Structure for the given 
%                           NTc chirp period values, NB bandwidths, Nc
%                           CN0s, Nm number of constellation sizes and Nvis
%                           values of visible satellites
%       outt.MU.tau     =   time-delay RMSE for the different satellites in
%                           the constellation
%
%--------------------------------------------------------------------------
%   @version    $3.0$
%   @date       $8-jul-2021$ $10-jun-2021$ $04-oct-2020$
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    outt    =   RMSE_Analysis(sett,flgt)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    %
    fsens   =   flgt.fsens;
    M       =   sett.M;
    if( M == 1 )
        flgt.Muser = 0;
    end
    flg     =   flgt.Muser;
    if flgt.fMUI
        MUI     =   sett.estimation.MUI;
    end    
    
    Mvis    =   sett.Mvis;
    B       =   sett.sig.B;  
    Tc      =   sett.sig.Tc;
    CN0     =   sett.estimation.CN0;
    %-  Zero-Padding factor for the FFT, to ease the RMSE measure of "low"
    %   accuracies
    zp      =   sett.estimation.zpfact;
    if( length(zp) == 1 )
        zp  =   zp*ones(length(CN0),1);
    end
    %
    fsim    =   sett.estimation.fsim;
%     acq     =   sett.estimation.acq;
%     if acq
%         th  =   sett.estimation.th;
%     end
    R       =   sett.estimation.R;
    if( length(R) == 1 )
        R   =   R*ones(length(CN0),1).';
    end
    const   =   sett.estimation.const;
    %
    NB      =   sett.samp.NB; 
    NTc     =   sett.samp.NTc;
    Nc      =   sett.samp.Nc;
    %
    tmp     =   sett;
    %
    Nm      =   length(M);
    Nvis    =   length(Mvis);
    CN0l    =   10.^(CN0/10);
    %
%----------------------------- RMSE ANALYSIS -----------------------------%
    %
    disp(['Accuracy analysis with ' num2str(R)]); 
    disp('statistical realizations');
    disp('%%%%%%%%%%');
    %
    %       Loops
    %
    RMSE_tau        =       nan(NTc,NB,Nc,Nm,Nvis);
    RMSE_fD         =       nan(NTc,NB,Nc,Nm,Nvis);
    %
    %
    if flgt.fMUI
        CRLBma      =       nan(NTc,NB,Nc);
        CRLBmi      =       nan(NTc,NB,Nc);
        CRLBavg     =       nan(NTc,NB,Nc);
    end
    CRLB            =       nan(NTc,NB,Nc);
    CRLBf           =       nan(NTc,NB,Nc);
    for ii = 1:NTc
        %-                  Loop NTc
        %
        tmp.samp.Nnc    =   1;
        tmp.sig.Tc      =   Tc(ii);
        disp(['* Tc = ' num2str(Tc(ii)*1e3) ' ms']);
        %
        for jj = 1:NB
            %-                  Loop NB
            %
            tmp.sig.B   =   B(jj);
            disp(['* Bandwidth = ' num2str(B(jj)/1e6) ' MHz']);
            %
            for k1 = 1:Nm
                %-              Loop Nm
                %
                tmp.M   =   M(k1);
                disp(['* M = ' num2str(M(k1)) ' sats.']);
                %
                [nset,~]    =   initParams(tmp,flgt);
                if fsim
                    nset.flg    =   1;
                    out         =   genBOKchirp_MultiUser(nset,flg);
                end
                %
                for k2 = 1:Nvis
                    %-          Loop Nvis
                    %
                    nset.Mvis    =   Mvis(k2);
                    disp(['* Mvis = ' num2str(Mvis(k2)) ' sats.']);
                    %
                    for kk = 1:Nc
                        %-      Loop Nc
                        %
                        if fsens
                            nset.estimation.zpfact =    zp(ii);
%                             if acq
%                                 nset.estimation.th      =   th(ii);
%                             end
                            nset.estimation.CN0    =   CN0(ii);
                            nset.estimation.R       =   R(ii);
                            disp(['* CN0  = ' num2str(CN0(ii)) ' dBHz']);
                        else
                            nset.estimation.zpfact =    zp(kk);
%                             if acq
%                                 nset.estimation.th      =   th(kk);
%                             end
                            nset.estimation.CN0    =   CN0(kk);
                            nset.estimation.R       =   R(kk);
                            disp(['* CN0  = ' num2str(CN0(kk)) ' dBHz']);
                        end
                        %
                        %-  Perform numerical simulations
                        %
                        if fsim
                            [tau,fD]  =   RMSE_Analysis_fnct(nset,out,flgt);
                            %
                            RMSE_tau(ii,jj,kk,k1,k2)    =   mean(tau);
                            RMSE_fD(ii,jj,kk,k1,k2)     =   mean(fD);
                            if const
                                MU(ii,jj,kk,k1,k2).tau  =   tau;
                            end
                        end
                        %
                        
                        %
                        %-      Loop Nc
                    end
                    %
                    %-          Loop Nvis
                end
                %
                %-              Loop Nm
            end
            if flgt.fMUI
                MUIavg              =   MUI.avg(ii,jj,:);
                MUIma               =   MUI.max(ii,jj,:);
                MUImi               =   MUI.min(ii,jj,:);
            end
            %-  Link with sensitivity analysis
            %
            if fsens
                CRLB(ii,jj,:)    =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*CN0l(ii)));
                CRLBf(ii,jj,:)   =   3./(CN0l(ii).*((2*pi.*Tc(ii)).^2).*Tc(ii));
                if flgt.fMUI                    
                    CRLBma(ii,jj,:)     =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*(CN0l(ii)/MUIma)));
                    CRLBmi(ii,jj,:)     =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*(CN0l(ii)/MUImi)));
                    CRLBavg(ii,jj,:)    =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*(CN0l(ii)/MUIavg)));
                else
                    % To Be Completed
                end
            else
            %
                CRLB(ii,jj,:)       =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*CN0l));                
                CRLBf(ii,jj,:)      =   3./(CN0l.*((2*pi.*Tc(ii)).^2).*Tc(ii));
                if flgt.fMUI
                    CRLBma(ii,jj,:)     =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*(CN0l/MUIma)));
                    CRLBmi(ii,jj,:)     =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*(CN0l/MUImi)));
                    CRLBavg(ii,jj,:)    =   (3./(2*pi*Tc(ii)).^2).*(Tc(ii)./((B(jj).^2).*(CN0l/MUIavg)));
                end
            end
            %
            %-                  Loop NB
        end
        
        %
        %-                      Loop NTc
    end
    if fsens
        Nc  =   NTc;
    end

    outt.tau        =   RMSE_tau;
    outt.fD         =   RMSE_fD;
    outt.CRLB       =   CRLB;
    outt.CRLBf      =   CRLBf;
    if flgt.fMUI
        outt.CRLBavg    =   CRLBavg;
        outt.CRLBma     =   CRLBma;
        outt.CRLBmi     =   CRLBmi;
    end
    if const
        outt.MU     =   MU;
    end
    
    
    
    
    %
    






%
end