% function [Pr,Pfa,h,tmp]     =   Pfa_Analysis_tmp(Nnc,B,Pn,Ps,R,Ck,...
%                                                   dchirp,ffit,Mvis)
%
%   Description:
%       Pfa analysis based on the input configuration parameters. The
%       generation of hypothesis H0 (no signal is present) is considered.
%       That is, we consider the acquisition of a given satellite, idsat1,
%       which is not present. But, we consider the presence of Mvis - 1
%       satellite with power Ps. Then the acquisition of idsat1 is
%       performed based on the received signal. The Pfa is numerically
%       computed over R Monte-Carlo realizations and its statistical model
%       is fitted to a Generalized Extreme Value distribution.
%
%   Inputs:
%       Nnc     =   Number of non-coherent integrations
%
%       B       =   Signal bandwidth (Hz)
%
%       Pn      =   Noise power (W)
%       Ps      =   Signal power (W)
%       R       =   Statistical (Monte-Carlo) realizations
%       Ck      =   (MxN) matrix with the I/Q samples of the BOK-chirp signal
%                   at each epoch and for each satellite in the constellation. 
%                   A total of N epochs and M satellites are considered                                                    
%       dchirp  =   De-Chirp local replicas
%
%       dchirp.ck_dc/n(1/2):    (MxN) matrix with the I/Q samples at each 
%                               epoch of the corresponding de-chirp signal
%                               for the positive/negative chirp components 
%                               of each satellite. The (1/2) corresponds to
%                               the (first/second) half of the chirp period
%       ffit    =   Statistical fitting option to perform fitting [1] or
%                   not [0] (if available)
%       Mvis    =   Number of visible satellites
%
%   Outputs:    
%       Pr      =   (Rx1) vector with the values of the acquisition
%                   variable in R Monte-Carlo runs
%       Pfa     =   (100x1) vector the Pfa values for different threshold
%                   values. The Pfa is numerically computed with R
%                   Monte-Carlo runs
%       h       =   (100x1) vector with the threshold values used to 
%                   numerically compute Pfa   
%       tmp     =   Results for the distribution fitting process
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $8-jul-2021$ $19-nov-2020$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function [Pr,Pfa,h,tmp]     =   Pfa_Analysis_tmp(Nnc,B,Pn,Ps,R,Ck,dchirp,ffit,Mvis)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    %
    M           =   size(Ck,1);                                             %   Constellation size
    N           =   size(dchirp.ck_dc1,2);                                  %   Number of signal samples
    Ncoh        =   N/Nnc;                                                  %   Number of samples corresponding to the coherent time
    Nfft        =   2^(nextpow2(N));                                        %   Number of samples to perform FFT
    %
    idsat1      =   dchirp.idsat;                                           %   Satellite id to be analyzed
    %-  De-Chirp local replicas
    %
    ck_dc.dc1   =   dchirp.ck_dc1(idsat1,:);
    ck_dc.dc2   =   dchirp.ck_dc2(idsat1,:);
    ck_dc.dcn1  =   dchirp.ck_dcn1(idsat1,:);
    ck_dc.dcn2  =   dchirp.ck_dcn2(idsat1,:);       
    %
    %-  Initialization of specific variables
    %
    Pr          =   zeros(R,1);
    perc        =   1;
    St          =   0;
    %
%----------------------------- MONTE-CARLO REALIZATIONS ------------------%
    for ii = 1:R
        %
        %-  Get "interferers" satellites
        %
        idsat   =   randperm(M,Mvis-1).';
        while( ~isempty(find(idsat == idsat1,1)) )% || ~isempty(find(idsat == (M-idsat1+2),1)) || ~isempty(find(idsat == 1,1)) )
            idsat   =   randperm(M,Mvis-1).';
        end
        %
        %-  Get signal with the given power in Ps
        %
        Cr      =   sum(Ck(idsat,:),1);
        Prx     =   (Cr*Cr')/length(Cr);
        Cr      =   sqrt((Mvis-1)*Ps/Prx)*Cr;
        %
        if( (ii/R)*100 >= 10*perc )
            disp(['> ' num2str(10*perc) ' % ...']);
            perc    =   perc + 1;
        elseif( ii == R )
            disp('> Done!');
            disp('% ...');
        end
        %     
        %-  Received signal with noise
        w           =   (sqrt(Pn/2))*(randn(1,N) + 1j*randn(1,N));
        r           =   Cr + w;
        %
        %-  Execute CSS acquisition
        %
        [Prt,S1,~]  =   acqCSS(r,ck_dc,Nnc,Ncoh,B,2); 
        Pr(ii)      =   Prt;
        %
        St          =   St + S1.S.^2;
        %
    end
    %
    %-  Get Pfa
    %
    m       =   mean(Pr);
    s       =   std(Pr);
    h       =   linspace(m-2*s,m+5*s,100);
    Nh      =   length(h);
    Pfa     =   zeros(Nh,1);
    for ii = 1:Nh
        Pfa(ii)     =   sum(Pr >= h(ii))/length(Pr);
    end
    %
    %-  Distribution fiting
    %
    warning('off','all');
    tmp     =   fitdist(Pr,'GeneralizedExtremeValue');
    warning('on','all');
    Pfat    =   1 - cdf('Generalized Extreme Value',h,tmp.k,tmp.sigma,tmp.mu);
    %
    %-  Plots
    %
    if( ffit )
        figure;semilogy(h,Pfa);
        hold on;semilogy(h,Pfat,'--');
        xlabel('Detection Threshold');
        ylabel('Prob. False Alarm');
        legend('Experimental','Semi-Analytical');
        title(['[B,Tc,CN0] = [' num2str(B/1e6) ' MHz,' num2str(N/B*1e3) ' ms,' num2str(10*log10((Ps/Pn)*B)) ' dBHz]']);
        grid
        %
        ttt         =   figure;
        th          =   histogram(Pr,100,'Visible','off');
        
        pdf_hist    =   th.Values/sum(th.Values);
        b           =   th.BinEdges(2:end);    
        df          =   b(2)-b(1);
        close(ttt);
        %
        %
        pdf_sa      =   df*gevpdf(b,tmp.k,tmp.sigma,tmp.mu);
        %
        figure;bar(b,pdf_hist);
        hold on;plot(b,pdf_sa,'LineWidth',2);
        xlabel('X_n');
        ylabel('pdf');
        title(['H_0 @ [B,Tc,CN0] = [' num2str(B/1e6) ' MHz,' num2str(N/B*1e3) ' ms,' num2str(10*log10((Ps/Pn)*B)) ' dBHz]']);
        legend('Experimental','Semi-Analytical');
        grid
        %
        %
        figure;plot(St/(Nfft*R));
        xlabel('Freq. index');
        ylabel('Power Spectral Density');
        title(['Up-chirp 1st de-chirp with noise @ [B,Tc,CN0] = [' num2str(B/1e6) ' MHz,' num2str(N/B*1e3) ' ms,' num2str(10*log10((Ps/Pn)*B)) ' dBHz]']);
        grid
    end
    %







end