% function    outt    =   Sensitivity_Analysis(sett,flg,flgt)
%
%   Description:
%       Main file for the sensitivity analysis with the selected
%       configuration in setParams.m. The current version is devoted to
%       perform the optimization of either the chirp period or the CN0.
%       This optimization finds the minimum parameter (i.e. Tc or CN0)
%       needed to obtain the target sensitivity performance with all the
%       other parameters fixed as given in setParams.m. The target
%       sensitivity performance is defined as a probability of detection >
%       0.9 with a probability of false alarm equal to the value set in
%       setParams.m, i.e. sett.sens.Pfa.
%
%   Inputs:
%       sett    =   Structure with setting parameters (see setParams.m)
%
%       flg     =   Structure with flag options (see setParams.m)
%
%   Outputs:   
%       outt    =   Structure with the results of the sensitivity analysis
% (opt = 2)
%       outt.sens.Tcopt.Tc:     Minimum chirp period to get target
%                               sensitivity performance
%       outt.sens.Tcopt.Pd:     Corresponding probability of detection
% (opt = 1)
%       outt.sens.CN0opt.CN0:   Minimum CN0 to get target sensitivity
%                               performance
%       outt.sens.CN0opt.Pd:    Corresponding probability of detection
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $7-jul-2021$ $21-may-2021$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    outt    =   Sensitivity_Analysis(sett,flg)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    
    Tc      =   sett.sig.Tc;

%----------------------------- GET OPTIMIZED MEASURE ---------------------%
    switch sett.sens.opt
        %-  Find Tc_min for given CN0
        %
        case 2      
            %
            Tcmin                   =   sett.sens.Tmin;                     %   Get minimum chirp period to be checked
            Tcmax                   =   sett.sens.Tmax;                     %   Get maximum chirp period to be checked
            %--     Get initial guess
            %
            if( sett.sens.flg_nc )  % Perform non-coherent integration
                Tc0                 =   Tc;
            else
                Tc0                 =   sett.sens.T0;
            end
            %
            %--     Run optimization engine
            %
            [Tc,Pd]                 =   getMeasmin(sett,flg,2,Tc0,Tcmin,...
                                                    Tcmax);
            %
            out.sens.Tcopt.Tc       =   Tc;
            out.sens.Tcopt.Pd       =   Pd;
        %
        %-  Find CN0_min for given Tc
        %
        case 1      
            %
            CN0min  =   sett.sens.CN0min;
            CN0max  =   sett.sens.CN0max;            
            CN00    =   sett.sens.CN00;
            %--     Run optimization engine
            %
            [CN0,Pd]    =   getMeasmin(sett,flg,1,CN00,CN0min,CN0max);
            %
            out.sens.CN0opt.CN0     =   CN0;
            out.sens.CN0opt.Pd      =   Pd;
            %
        %
        otherwise
%             %
%             if ~flgt
%                 Pd              =   zeros(NTc,Nc,NB);
%                 Pdc             =   zeros(NTc,Nc,NB);
%                 for ii = 1:NB
%                     tmp.sig.B       =   B(ii);
%                     disp(['* Bandwidth = ' num2str(B(ii)/1e6) ' MHz']);
%                     for kk = 1:Nc
%                         a           =   zeros(2,NTc);
%                         b           =   zeros(2,NTc);
%                         tmp.sens.CN0    =   CN0(kk);
%                         disp(['* CN0  = ' num2str(CN0(kk)) ' dBHz']);
%                         for jj = 1:NTc
%                             %               
%                             tmp.sig.Tc      =   Tc(jj);   
%                             disp(['* Tc = ' num2str(Tc(jj)*1e3) ' ms']);
%                             [nset,flg]      =   initParams(tmp,flg);
%                             %
%                             out.sens        =   Sensitivity_Analysis_fcnt(nset,flg);
%                             hsp           =   out.sens.hsp;
%                             hc           =   out.sens.hcell;
%                             Pd(jj,kk,ii)    =   1 - cdf('Generalized Extreme Value',hsp,out.sens.fitPd.k,out.sens.fitPd.sigma,out.sens.fitPd.mu);
%                             Pdc(jj,kk,ii)   =   1 - cdf('Generalized Extreme Value',hc,out.sens.fitPd.k,out.sens.fitPd.sigma,out.sens.fitPd.mu);
%                             if flg.plot.fsens
%                                 sensResults;
%                             end
%                         end
%                     end
%                 end
%             else
%                 out.sens        =   Sensitivity_Analysis_fcnt(sett,flg);
%                 if flg.plot.fsens
%                     ii          =   1;
%                     jj          =   1;
%                     kk          =   1;
%                     NTc         =   sett.samp.NTc;
%                     Tc          =   sett.sig.Tc;
%                     B           =   sett.sig.B;
%                     CN0         =   sett.sens.CN0;
%                     sensResults;
%                 end                    
%             end
%             %
    end
    %
    outt                =   out.sens;
    %
end