% function    outt    =   Sensitivity_Analysis_fcnt(set,flg,varargin)
%
%   Description:
%       Sensitivity analysis, including both the analysis of the 
%       probabilities of detection and false alarm. The ultimate goal is to
%       get the detection threshold needed to obtain Pfa = set.sens.Pfa,
%       and then to compute the Pd with such detection threshold. The
%       results for the analysis of these probabilities are stored for
%       further use in the future. This is for the sake of time-consumption
%       of the sensitivity analysis. We consider the analysis for a given
%       satellite, similar results are obtained for different satellites.
%
%   Inputs:
%       set         =   Structure with setting parameters (see
%                       setParams.m), but considering only one value of
%                       bandwidth, chirp period and CN0
%
%       flg         =   Structure with flag options (see setParams.m)
%
%       varargin    =   Variable-length input argument list
%
%           Ck      =   (MxN) matrix with the I/Q samples of the BOK-chirp
%                       signal at each epoch and for each satellite in
%                       view. A total of N epochs and M the number of
%                       satellites in the constellation
%                               
%           dchirp  =   De-Chirp local replicas
%
%           dchirp.ck_dc/n(1/2):    (MxN) matrix with the I/Q samples at  
%                                   each epoch of the corresponding  
%                                   de-chirp signal for the positive/negative 
%                                   chirp components of each satellite. The
%                                   (1/2) corresponds to the (first/second)
%                                   half of the chirp period
%
%   Outputs:    
%       outt        =   Structure with the results of the sensitivity
%                       analysis
%
%       outt.Pfah   =   (1x100) vector with the Pfa values for different
%                       values of the detection threshold, given in outt.h.
%                       The fitted model is used
%       outt.Pdh    =   (1x100) vector with the Pd values for different
%                       values of the detection threshold, given in outt.h.
%                       The fited model is used
%       outt.h      =   (1x100) vector with the detection threshold values
%                       used to compute Pfah and Pdh
%       outt.hsp    =   Detection threshold value to obtain a Pfa =
%                       set.sens.Pfa
%       outt.fitP   =   Parameters for the distribution fitting of the Pfa
%       outt.fitPd  =   Parameters for the distribution fitting of the Pd
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $8-jul-2021$ $19-nov-2020$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    outt    =   Sensitivity_Analysis_fcnt(set,flg,varargin)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    %
    M       =   set.M;                                                      %   Constellation size
    Mvis    =   set.Mvis;                                                   %   Number of visible satellites
    B       =   set.sig.B;                                                  %   Bandwidth
    T       =   set.sig.Tc;                                                 %   Chirp period
    R       =   set.sens.R;                                                 %   Number of statistical realizations for the Pfa analysis
    Rd      =   set.sens.Rd;                                                %   Number of statistical realizations for the Pd analysis
    CN0     =   set.sens.CN0;                                               %   CN0 value 
    alpha   =   set.sens.Pfa;                                               %   Pfa target value
    ffit    =   set.sens.fit;                                               %   Statistical fitting option
    Nnc     =   set.samp.Nnc;                                               %   Number of non-coherent integration
    %
    N       =   Nnc*ceil(B*T);                                              %   Number of signal samples
    %-  Get variable-length input argument list
    %
    if( ~isempty(varargin) )        % Signal already available       
        Ck      =   varargin{1};
        dchirp  =   varargin{2};
        fgen    =   0;
    else                            % Signal to be generated
        fgen    =   1;        
    end
    %
    %-  Control plot flags
    %
    if flg.graphics
        if flg.plot.fsens
            fit_flg     =   flg.plot.sens.fit;
        else
            fit_flg     =   0;
        end
    else
        fit_flg         =   0;
    end
    %
%----------------------------- SENSITIVITY ANALYSIS ----------------------%    
%
    %-  Pfa Analysis
    %
    %--     Get sting with configuration parameters
    %
    if( Nnc > 1 )
        str         =   ['Sensitivity/Semi_Analytical/Pfa_CN0' num2str(CN0) 'dBHz_Tc' num2str(T*1000) 'ms_Nnc' num2str(Nnc) '_B' num2str(B/1e6) 'MHz_M' num2str(M) '_Mvis' num2str(Mvis) '.mat'];
    else
        str         =   ['Sensitivity/Semi_Analytical/Pfa_CN0' num2str(CN0) 'dBHz_Tc' num2str(T*1000) 'ms_B' num2str(B/1e6) 'MHz_M' num2str(M) '_Mvis' num2str(Mvis) '.mat'];
    end
    %
    %--     Results already exist?
    %
    if( exist(str,'file') == 0 || ffit == 1)
        disp('Generating BOKchirp signal:');
        disp('% ...');
        %
        set.flg     =   0;
        if fgen
            %---    We fix the satellite to be analyzed so that avaaged
            %       results for the same satellite are considered.
            %       Time-delay and Doppler frequency of such satellite set
            %       to 0, the rest of satellites have a random time-delay
            %       and Doppler profile
            %
            idsat   =   1;
%             idsat           =   randperm(M,1).';
%             while( idsat == 1 )
%                 idsat   =   randperm(M,1).';
%             end
            set.tau(idsat)  =   0;
            set.fD(idsat)   =   0;
            %---    Generate signal
            %
            out             =   genBOKchirp_MultiUser(set,flg.Muser); 
            %
        else
            idsat           =   1;
%             idsat           =   dchirp.idsat;
        end
        %---    Initialize specific parameters
        %
        initParams_NoiseAnalysis;         
        Nfft            =   2^(nextpow2(N));
        %
        disp(['Pfa Analysis with ' num2str(R)]); 
        disp('statistical realizations');
        disp('%%%%%%%%%%');
        %
        %---    Get Pfa results: Detection threshold hc
        %
%         [~,~,ht,tmp]  =   Pfa_Analysis(Nnc,B,CN0,R,set,flg.Muser,fit_flg,Mvis);
        [~,~,ht,tmp]  =   Pfa_Analysis_tmp(Nnc,B,Pn,Ps,R,Ck,dchirp,fit_flg,Mvis);
        %
        fitP.k      =   tmp.k;
        fitP.sigma  =   tmp.sigma;
        fitP.mu     =   tmp.mu;      
        func            =   @(x) (1 - cdf('Generalized Extreme Value',x,fitP.k,fitP.sigma,fitP.mu)) - alpha;
        hc              =   fzero(func,0);
        %
        
        %---    Get Pd results: Distribution fiting
        %
        disp(['Pd Analysis with ' num2str(Rd)]); 
        disp('statistical realizations');
        disp('%%%%%%%%%%');
        %
        [Pd,~,tmp]     =   Pd_Analysis_tmp(Nnc,B,Pn,Ps,Rd,Ck,dchirp,fit_flg,hc,Mvis);
%         [Pd,~,tmp]      =   Pd_Analysis(Nnc,B,CN0,Rd,set,flg.Muser,fit_flg,hc,Mvis);
        fitPd.k         =   tmp.k;
        fitPd.sigma     =   tmp.sigma;
        fitPd.mu        =   tmp.mu;
        %
        %---    Save results
        %
        save(str,'ht','fitP','fitPd');
        %
    else
        %---    Load results
        load(str,'ht','fitP','fitPd');
        Nfft            =   2^(nextpow2(N));
        func            =   @(x) (1 - cdf('Generalized Extreme Value',x,fitP.k,fitP.sigma,fitP.mu)) - alpha;
        hc              =   fzero(func,0);
        %
        Pd              =   1 - cdf('Generalized Extreme Value',hc,fitPd.k,fitPd.sigma,fitPd.mu);
        %   
        if fgen
            Ck  =   [];
            dchirp  =   [];
        end
    end
    %
    Pfah            =   1 - cdf('Generalized Extreme Value',ht,fitP.k,fitP.sigma,fitP.mu);
    Pdh             =   1 - cdf('Generalized Extreme Value',ht,fitPd.k,fitPd.sigma,fitPd.mu);
    %  
%----------------------------- SENSITIVITY OUT STRUCTURE -----------------%
    outt.Pd     =   Pd;
    outt.Pfah   =   Pfah;
    outt.Pdh    =   Pdh;
    outt.h      =   ht;
    outt.hsp    =   hc;
    outt.fitP   =   fitP;
    outt.fitPd  =   fitPd;
    outt.Nfft   =   Nfft;
    %
    if fgen
        outt.Ck         =   Ck;
        outt.dchirp     =   dchirp;
    end
    %












end