% function    [Mopt,Pd]      =   getMeasmin(sett,flgs,flg,M0,Mmin,Mmax)
%
%   Description:
%       Execution loops for the sensitivity analysis with the selected
%       input configuration. Is used to initialize the parameters needed to
%       properly execute the optimization proccess of the sensitivity
%       analysis. The initialization depends on the input configuration. If 
%       the results for the input configuration have already generated
%       (sometime), they are loaded. Otherwise, the results are generated
%
%   Inputs:
%       sett    =   Structure with setting parameters (see setParams.m)
%
%       flgs     =   Structure with flag options (see setParams.m)
%
%       flg     =   Optimization flag for CN0 minimization [1] or chirp
%                   period minimization [2]
%
%       M0      =   Average value (guess) of the optimization process
%
%       Mmin    =   Minimum value (guess) of the optimization process
%
%       Mmax    =   Maximum value (guess) of the optimization process
%
%   Outputs:    
%       Mopt    =   Optimized measure, i.e., minimum measure that achieves
%                   target sensitivity performance for the given parameters
%
%       Pd      =   Probability of detection when using Mopt. This Pd > 0.9
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $8-jul-2021$ $21-may-2021$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    [Mopt,Pd]      =   getMeasmin(sett,flgs,flg,M0,Mmin,Mmax)
%
%----------------------------- PARAMETER INITIALIZATION ------------------%
    %
    M       =   sett.M;                                                     % Constellation size (i.e., number of satellites)
    Mvis    =   sett.Mvis;                                                  % Visible satellites
    B       =   sett.sig.B;                                                 % Signal bandwidth  
    NB      =   sett.samp.NB;                                               % Number of bandwidth values to be simulated       
    %
    tmp     =   sett;
    %
    M0t     =   M0;
    Mmint   =   Mmin;
    Mmaxt   =   Mmax;    
    %
    switch flg
        case 1  % CN0 optimization
            %
            NTc     =   sett.samp.NTc;
            Tc      =   sett.sig.Tc;
            %
            Ni      =   NTc;
            Nj      =   NB;            
            %
        case 2  % Tc optimization
            %
            Nc      =   sett.samp.Nc;
            CN0     =   sett.sens.CN0;
            %
            Ni      =   NB;
            Nj      =   Nc;              
            %
    end
    %
    %
    Nk1     =   length(M);                                                  % # of constellation size values to be analyzed
    Nk2     =   length(Mvis);                                               % # of visible satellite values to be analyzed
    %-  Initialize output variables
    %
    Mopt    =   zeros(Ni,Nj,Nk1,Nk2);
    Pd      =   zeros(Ni,Nj,Nk1,Nk2); 
    %
%----------------------------- LOOPS -------------------------------------%
    %
    for ii = 1:Ni
        %
        tmp.samp.Nnc    =   1;
        switch flg
            case 1
                %
                tmp.sig.Tc      =   Tc(ii);  
                disp(['* Tc = ' num2str(Tc(ii)*1e3) ' ms']);
                %
                if( length(M0t) == 1 )
                    M0      =   M0t;
                else
                    M0      =   M0t(ii);
                end
                if( length(Mmint) == 1 )
                    Mmin    =   Mmint;
                else
                    Mmin    =   Mmint(ii);
                end
                if( length(Mmaxt) == 1 )
                    Mmax    =   Mmaxt;
                else
                    Mmax    =   Mmaxt(ii);
                end
                %
            case 2
                %
                tmp.sig.B   =   B(ii);
                disp(['* Bandwidth = ' num2str(B(ii)/1e6) ' MHz']);
                %
        end
        %
        for jj = 1:Nj
            %
            switch flg
                case 1
                    tmp.sig.B   =   B(jj);
                    disp(['* Bandwidth = ' num2str(B(jj)/1e6) ' MHz']);
                case 2
                    %
                    tmp.sens.CN0    =   CN0(jj);
                    disp(['* CN0  = ' num2str(CN0(jj)) ' dBHz']);
                    %
                    if( length(M0t) == 1 )
                        M0      =   M0t;
                    else
                        M0      =   M0t(jj);
                    end
                    if( length(Mmint) == 1 )
                        Mmin    =   Mmint;
                    else
                        Mmin    =   Mmint(jj);
                    end
                    if( length(Mmaxt) == 1 )
                        Mmax    =   Mmaxt;
                    else
                        Mmax    =   Mmaxt(jj);
                    end
                    %
            end
            %
            for k1 = 1:Nk1
                %
                fgen        =   1;
                tmp.M           =   M(k1);
                disp(['* M = ' num2str(M(k1)) ' sats.']);
                %
                for k2 = 1:Nk2
                    %
                    str         =   [];
                    tmp.Mvis    =   Mvis(k2);
                    disp(['* Mvis = ' num2str(Mvis(k2)) ' sats.']);
                    diro        =   [pwd '/Sensitivity/Opt/'];
                    switch flg
                        case 1
                            dir     =   [diro 'CN0opt/Tc' num2str(Tc(ii)*1e3) 'ms'];
                            str     =   ['Tc' num2str(Tc(ii)*1e3) 'ms_B' ...
                                         num2str(B(jj)/1e6) 'MHz_M' ...
                                         num2str(M(k1)) '_Mv' num2str(Mvis(k2))];
                        case 2     
                            if( tmp.sens.flg_nc )
                                diro    =   [diro 'Nncopt/'];
                                str     =   ['Tc' num2str(tmp.sig.Tc*1e3) ... 
                                             'ms_B' num2str(B(ii)/1e6) ...
                                             'MHz_CN0' num2str(CN0(jj)) ...
                                             '_M' num2str(M(k1)) '_Mv' ...
                                             num2str(Mvis(k2))];
                            else
                                diro    =   [diro 'Tcopt/'];
                                str     =   ['B' num2str(B(ii)/1e6) 'MHz_CN0' ...
                                         num2str(CN0(jj)) '_M' num2str(M(k1)) ...
                                         '_Mv' num2str(Mvis(k2))];
                            end
                            dir         =   [diro 'B' num2str(B(ii)/1e6) 'MHz'];                            
                    end   
                    %
                    strr    =   [dir '/out_Server_' str '.mat'];
                    %-  Check if results for given configuration already
                    %   exist (generated externaly, e.g., in server)
                    %
                    if( exist(strr,'file') == 2 )   % Load results
                        A                   =   load(strr);
                        Mopt(ii,jj,k1,k2)   =   A.out.sens.Tcopt.Tc;
                        Pd(ii,jj,k1,k2)     =   A.out.sens.Tcopt.Pd;
                        A.out.sens.Tcopt.Pd                        
                    else                            %   Generate results
                        if fgen % Generate signal
                            [Pdt,meas]  =   getoptmeas(M0,Mmax,Mmin,...
                                                        tmp,flgs,flg);

                        else    %   Signal already generated (in dchirp)
                            [Pdt,meas]  =   getoptmeas(M0,Mmax,Mmin,...
                                                        tmp,flgs,flg,Ck,...
                                                        dchirp);
                        end                    
                        Ck                  =   meas.Ck;
                        dchirp              =   meas.dchirp; 
                        Mtmp                =   M0;
                        M0                  =   meas.m0;
                        fgen                =   0;
                        %
                        Mopt(ii,jj,k1,k2)   =   M0;
                        Pd(ii,jj,k1,k2)     =   Pdt;
                        if( (flg == 2 && tmp.sens.flg_nc) )                        
                            M0              =   Mtmp;
                            if((flg == 2 && tmp.sens.flg_nc) )
                                tmp.samp.Nnc    =   meas.Nnc;
                            end
                        end 
                        out.sens.Tcopt.Tc = Mopt(ii,jj,k1,k2);
                        out.sens.Tcopt.Pd = Pd(ii,jj,k1,k2);
                        save(strr,'out');
                    
                    end                                      
                    %
                end
            end
        end
    end
end