% function    [Pd,meas]   =   getoptmeas(meas0,measmax,measmin,tmp,flg,...
%                                       opt,varargin)
%
%   Description:
%       Execution of the optimization process used to obtain the minimum
%       measure needed to achieve the target sensitivity performance (i.e.
%       Pd > 0.9 @ Pfa = tmp.sens.Pfa). For that, we get the analysis of
%       Pfa and Pd from the Sensitivity_Analysis_fnct. In particular, we
%       get the detection threshold needed to obtain Pfa = tmp.sens.Pfa,
%       and then we obtain Pd with such detection threshold. Then, the
%       optimization process starts: If Pd < 0.9 we increase our guess for
%       the optimized value, whereas if Pd >= 0.95 we decrease our guess.
%       We repeat this process until 0.9 < Pd <= 0.95, and then we end the    
%       process by providing the optimized measure as the value of the last
%       guess.
%
%   Inputs:
%       meas0       =   Average value (guess) of the optimization process
%
%       measmax     =   Maximum value (guess) of the optimization process
%
%       measmin     =   Minimum value (guess) of the optimization process
%
%       tmp         =   Structure with setting parameters (see
%                       setParams.m), but considering only one value of
%                       bandwidth, chirp period and CN0
%
%       flg         =   Structure with flag options (see setParams.m)
%
%       opt         =   Optimization flag for CN0 minimization [1] or chirp
%                       period minimization [2]
%
%       varargin    =   Variable-length input argument list
%           
%           Ck      =   (MxN) matrix with the I/Q samples of the BOK-chirp
%                       signal at each epoch and for each satellite in
%                       view. A total of N epochs and M the number of
%                       satellites in the constellation
%                               
%           dchirp  =   De-Chirp local replicas
%
%           dchirp.ck_dc/n(1/2):    (MxN) matrix with the I/Q samples at  
%                                   each epoch of the corresponding  
%                                   de-chirp signal for the positive/negative 
%                                   chirp components of each satellite. The
%                                   (1/2) corresponds to the (first/second)
%                                   half of the chirp period
%
%   Outputs:    
%       Pd      =   Optimized probability of detection (i.e., when using
%                   the optimized measure value)
%
%       meas    =   Feedback measures
%
%       meas.m0     =   Next initial measure
%       meas.Nnc    =   Number of non-coherent integrations
%       meas.Ck     =   Ck
%       meas.dchirp =   dchirp
%
%--------------------------------------------------------------------------
%   @version    $2.0$
%   @date       $8-jul-2021$ $21-may-2021$   
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    [Pd,meas]   =   getoptmeas(meas0,measmax,measmin,tmp,flg,opt,...
                                        varargin)
    %
%----------------------------- PARAMETER INITIALIZATION ------------------%
    if( ~isempty(varargin) )        
        Ck      =   varargin{1};
        dchirp  =   varargin{2};
        if( isempty(Ck) )
            fgen    =   1;
        else
            fgen    =   0;
        end
    else
        fgen    =   1;      
    end
    %
    Nit         =   0;
    fext        =   0;
    %
    if( tmp.sens.flg_nc && opt == 2 )   % Non-Coherent integration optimization
        Tc      =   meas0;
        Tcmin   =   measmin;
    end
    %
    Nnc         =   tmp.samp.Nnc;
    Nnct        =   [];
    %
%----------------------------- GET OPTIMIZED MEASURE ---------------------%
    while( ~fext && Nit < 500)
        %
        if( opt == 1 )
            tmp.sens.CN0    =   meas0;
            disp(['* CN0 = ' num2str(meas0) ' dB-Hz']);
        elseif( opt == 2 )
            tmp.sig.Tc      =   meas0;
            disp(['* Tc = ' num2str(meas0*1e3) ' ms']);
        end        
        %
        tmp.samp.Nnc        =   Nnc;
        [nset,flg]          =   initParams(tmp,flg); 
        %-  Execute Sensitivity Analysis
        %
        if fgen
            out.sens        =   Sensitivity_Analysis_fcnt(nset,flg);
            Ck              =   out.sens.Ck;
            if( ~isempty(Ck) )
                dchirp      =   out.sens.dchirp;
                fgen        =   0;   
            else
                dchirp      =   [];
            end
        else
            out.sens    =   Sensitivity_Analysis_fcnt(nset,flg,Ck,dchirp);
        end
        %
        %-  Get detection threshold and corresponding Pd
        %
        hsp             =   out.sens.hsp;
        Pd              =   1 - cdf('Generalized Extreme Value',hsp,out.sens.fitPd.k,out.sens.fitPd.sigma,out.sens.fitPd.mu)
        %
        if( Pd < 0.9 ) %    Increase measure
            if( tmp.sens.flg_nc && opt == 2 ) % NCI optimization
                %
                Nnctt       =   Nnc;
                %
                measmin     =   Nnc*meas0;
                if( measmin == measmax )
                    measmax     =   2*measmax;
                end
                meas0       =   Nnc*meas0 + (measmax - Nnc*meas0)/2;
                Nnc         =   ceil(meas0/Tc);                
                meas0       =   Tc;
                if( Nnc == Nnctt )                    
                    if( ~isempty(find(Nnct == Nnc,1)))
                        fext    =   1;
                    else
                        Nnct    =   [Nnct Nnc];
                    end
                    Nnc     =   Nnc + 1;
                end
                disp(['* Nnc = ' num2str(Nnc)]);
            else
                measmin     =   meas0;
                if( measmin == measmax )
                    measmax     =   2*measmax;
                end
                meas0       =   meas0 + (measmax - meas0)/2;
            end
        elseif( Pd >= 0.95 ) %  Decrease measure
            if( tmp.sens.flg_nc && opt == 2 )   %   NCI optimization
                Nnct    =   [Nnct Nnc];
                Nnctt       =   Nnc;
                %
                measmax     =   Nnc*meas0;
                meas0       =   measmin + (Nnc*meas0 - measmin)/2;
                Nnc         =   ceil(meas0/Tc);
                meas0   =   Tc;
                if( Nnc == Nnctt )
                    Nnc     =   Nnc - 1;
                    if( ~isempty(find(Nnct == Nnc,1)) || Nnc == 0)
                        fext    =   1;
                    end
                    Nnct    =   [Nnct Nnc];
%                     if( Nnc == 1 )
%                         Nnct    =   [Nnct Nnc];
%                     end
                else
                    if( ~isempty(find(Nnct == Nnc,1)) || Nnc == 0)
                        fext    =   1;
                    end
                end
                disp(['* Nnc = ' num2str(Nnc)]);                
            else
                measmax     =   meas0;
                meas0       =   measmin + (meas0 - measmin)/2;
            end
        else    %   Exit process
            fext    =   1;                                
        end
        %
        if ~fext
            fgen    =   1;            
        else
            if nset.sens.flg_nc     % Non-coherent integration optimization
                if( Nnc == 0 )
                    Nnc     =   1;
                end
                meas.m0     =   Nnc*Tc;
                meas.Nnc    =   Nnc;
            else
                meas.m0     =   meas0;
            end
            meas.Ck     =   Ck;
            meas.dchirp =   dchirp;
        end
        %
        Nit     =   Nit + 1;
        %
    end
    
    
    
    
    %





end