% function    out     =   genBOKchirp(f0,mu,B,tau,fD,Tc,Nnc)
%
%   Description:
%       Generation of the output results for the BOK-chirp with the given
%       input parameters.
%
%   Inputs:
%       f0:     Initial frequency (Hz)
%       mu:     Chirp rate (Hz^2)
%       B:      Signal Bandwidth (Hz)
%       tau:    Time-Delay (sec.)
%       fD:     Frequency Doppler (Hz)
%       Tc:     Chirp period (sec.)
%       Nnc:    Number of non-coherent integrations 
%
%   Outputs: To Be Completed
%       out:    Structure with the results of the generated BOK-chirp
%               signal
%
%       .fkp/n:     (Nx1) vector with the values corresponding to the 
%                   signal frequency (Hz) at each epoch of the positive 
%                   (up-chirp)/negative (down-chirp) components. A total of  
%                   N = Nnc*ceil(B*Tc) epochs are considered
%       .fk_dc/n:   (Nx1) vector with the frequency of the de-chirp signal
%                   for the positive/negative chirp components
%       .phkp/n:    (Nx1) vector with the values corresponding to the phase 
%                   (rad) at each epoch of the positive/negative components
%       .ckp/n:     (Nx1) vector with the values corresponding to the I/Q 
%                   samples of the positive/negative chirp components
%       .Ck:        (Nx1) vector with the I/Q samples of the BOK-chirp
%                   signal for each epoch
%
%       For the following outputs and for the case of the MDS scheme, we
%       have 2 different versions of the following outputs; each one
%       corresponding to each half of the chirp period
%
%       .ck_dc/n:   (Nx1) vector with the I/Q samples at each epoch of the
%                   corresponding de-chirp signal of the positive/negative
%                   component
%       .ss_dc/n:   (Nx1) vector with the I/Q samples at each epoch of the
%                   corresponding de-chirped signal of the positive/negative
%                   component
%       .s_dc/n:    (Nx1) vector with the I/Q samples at each epoch of the
%                   de-chirp for the positive/negative branch of the
%                   BOK-chirp signal
%       
%---------------------------------------------------------------------------------------------------- 
%   @version    $2.0$
%   @date       $6-jul-2021$ $11-may-2020$ $17-apr-2020$ 
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function    out     =   genBOKchirp(f0,mu,B,tau,fD,Tc,Nnc)
%
    flg                         =   (size(mu,2) == 1);                      %   Multi-Dual slope scheme flag
    N                           =   Nnc*ceil(Tc*B);                         %   Number of chirp samples
    
%----------------------------- GENERATE POSITIVE SLOPE -------------------%
    %
    if flg
        [fkp,phkp,ckp]          =   genchirp(f0,mu,B,tau,fD,Tc,Nnc);
    else
        [fkp,phkp,ckp]          =   genMultiSlope2(mu,B,tau,fD,Tc,Nnc);
    end
    %
%----------------------------- GENERATE NEGATIVE SLOPE -------------------%
    %
    if flg
        [fkn,phkn,ckn]          =   genchirp(f0,-mu,B,tau,fD,Tc,Nnc);
    else
        [fkn,phkn,ckn]          =   genMultiSlope2(-mu,B,tau,fD,Tc,Nnc);
    end
    %
%---------------------------- GENERATE BOK-CHIRP SIGNAL ------------------%
    %
    Ck                          =   ckp + ckn;
    %
%---------------------------- GENERATE BOK-CHIRP RESULTS -----------------%
    %
    %-  De-Chirp process
    %
    %--     Positive (up-chirp) branch
    %
    if flg
        [fk_dc,~,ck_dc]         =   genchirp(0,-mu,B,0,0,Tc,Nnc);           %   Get up-chirp local replica (conjugated)
        %
        ss_dc                   =   ckp.*ck_dc;                             %   (up-chirp) De-Chirp for the up-chirp component
        s_dc                    =   Ck.*ck_dc;                              %   (up-chirp) De-Chirp for the BOK-chirp
    else
        [fk_dc,~,ck_dc]         =   genMultiSlope2(-mu,B,0,0,Tc,Nnc);      %   Get up-chirp local replica (conjugated)
        %
        Ncoh                    =   ceil(B*Tc);
        Nh                      =   floor(Ncoh/2);
        ck_dc1                  =   zeros(N,1);
        ck_dc2                  =   zeros(N,1);
        %
        %---    Get 1st and 2nd half part of the chirp period
        %
        for ii = 1:Nnc
            init                =   (ii-1)*Ncoh + 1;
            fin                 =   init + Nh - 1;
            ck_dc1(init:fin)    =   ck_dc(init:fin);
            ck_dc2(fin+1:end)   =   ck_dc(fin+1:end);
        end
        %
        %---    De-chirp for the up-chirp component (ss_dc1/2) and for the
        %       BOK-chirp signal (s_dc1/2)
        %
        ss_dc1                  =   ckp.*ck_dc1;
        ss_dc2                  =   ckp.*ck_dc2;
        s_dc1                   =   Ck.*ck_dc1;
        s_dc2                   =   Ck.*ck_dc2;
        %
    end
    %
    %--     Negative (down-chirp) branch
    %
    if flg
        [fk_dcn,~,ck_dcn]       =   genchirp(0,mu,B,0,0,Tc,Nnc);                    %   Get down-chirp local replica (conjugated)
        %
        %---    De-chirp for the down-chirp component (ss_dcn) and for the
        %       BOK-chirp signal (s_dc)
        %
        ss_dcn                  =   ckn.*ck_dcn;
        s_dcn                   =   Ck.*ck_dcn; 
    else
        [fk_dcn,~,ck_dcn]       =   genMultiSlope2(mu,B,0,0,Tc,Nnc);            %   Get down-chirp local replica (conjugated)
        %        
        ck_dcn1             =   zeros(N,1);
        ck_dcn2             =   zeros(N,1);
        ck_dcn1(1:Nh)       =   ck_dcn(1:Nh);
        ck_dcn2(Nh+1:end)   =   ck_dcn(Nh+1:end);
        %
        %---    De-chirp for the down-chirp component (ss_dcn1/2) and for 
        %       the BOK-chirp signal (s_dcn1/2)
        %
        ss_dcn1             =   ckp.*ck_dcn1;
        ss_dcn2             =   ckp.*ck_dcn2;
        s_dcn1              =   Ck.*ck_dcn1;
        s_dcn2              =   Ck.*ck_dcn2;
        %
    end
    %  
    %
%-------------------------------- SAVE OUT STRUCTURE ---------------------%
    %
    out.fkp         =   fkp;        
    out.phkp        =   phkp;
    out.ckp         =   ckp;
    out.fkn         =   fkn;
    out.phkn        =   phkn;
    out.ckn         =   ckn;
    out.Ck          =   Ck;
    %
    out.fk_dc       =   fk_dc;    
    out.fk_dcn      =   fk_dcn;
    
    %
    if flg
        out.ck_dc       =   ck_dc;
        out.ck_dcn      =   ck_dcn;
        %
        out.ss_dc       =   ss_dc;
        out.s_dc        =   s_dc;    
        out.ss_dcn      =   ss_dcn;
        out.s_dcn       =   s_dcn;
    else
        out.ck_dc1       =   ck_dc1;
        out.ck_dcn1      =   ck_dcn1;
        %
        out.ck_dc2       =   ck_dc2;
        out.ck_dcn2      =   ck_dcn2;
        %
        out.ss_dc1       =   ss_dc1;
        out.s_dc1        =   s_dc1;    
        out.ss_dcn1      =   ss_dcn1;
        out.s_dcn1       =   s_dcn1;        
        %
        out.ss_dc2       =   ss_dc2;
        out.s_dc2        =   s_dc2;    
        out.ss_dcn2      =   ss_dcn2;
        out.s_dcn2       =   s_dcn2;
    end








end