% function    out     =   genBOKchirp_MultiUser(set,flg)
%
%   Description:
%       Generation of the Multi-Satellite BOK-chirp signal with the
%       configuration given in set (see setParams.m) and for the 
%       multi-satellite scheme given by flg. The generation for different 
%       satallites depending on the number of satellites in the 
%       constellation (possibly variable) is considered. Time-Delay and 
%       frequency Doppler is also considered. 
%
%   Inputs:
%       set     =   Structure with setting parameters (see setParams.m)
%
%       flg     =   Multi-Satellite scheme flag: [0] for single-user, [1]
%                   for multi slope schme, [2] for multi-dual slope scheme
%                   and [3] for multi-frequency scheme
%
%   Outputs: 
%       out     =   Structure with the results of the multi-satellite
%                   BOK-chirp signal.
%
%       .BOK.sig.fkp/n:         (MxN) matrix with the values corresponding   
%                               to the signal frequency (Hz) at each epoch  
%                               of the positive (up-chirp)/negative (down-chirp) 
%                               components for each satellite. A total of N 
%                               epochs and M satellites are considered
%               .phkp/n:        (MxN) matrix with the values corresponding  
%                               to the phase (rad) at each epoch of the 
%                               positive/negative components for each satellite. 
%               .ckp/n:         (MxN) matrix with the I/Q samples of the 
%                               positive/negative chirp components for each
%                               satellite
%               .Ck:            (MxN) matrix with the I/Q samples of the 
%                               BOK-chirp signal at each epoch and for each
%                               satellite
%
%       .BOK.dchirp.fk_dc/n:    (MxN) matrix with the frequency of the 
%                               de-chirp signal for the positive/negative 
%                               chirp components of each user at each epoch
%                  .ck_dc/n:    (MxN) matrix with the I/Q samples at each 
%                               epoch of the corresponding de-chirp signal 
%                               for the positive/negative chirp components
%                               of each user
%                  .ss_dc/n:    (MxN) matrix with the I/Q samples at each 
%                               epoch of the corresponding de-chirped signal
%                               for the positive/negative chirp component
%                               of each user                   
%                  .s_dc/n:     (MxN) matrix with the I/Q samples at each 
%                               epoch of the de-chirp for the positive/negative 
%                               branch of the BOK-chirp signal of each user
%
% If Nm > 1 the previous outputs start by out.BOK.MU(ii) instead of out.BOK
% and it corresponds to the same outputs as described above but for a
% constellation size of M(ii) satellites (as configured in setParams.m).
% All the constellation sizes are analyzed
%
%       .BOK.MU.f0:             (Mx1) vector with the values of the initial
%                               frequency for each satellite, with M the
%                               number of satellites
%              .mu:             (Mx1) vector with the values of the chirp
%                               rate for each satellite
%              .Cr:             (Nx1) vector with the I/Q samples of the
%                               received signal (sum of M satellites)
%              .r_dc/n:         (Nx1) vector with I/Q samples of the
%                               de-chirped received signal
%                               (positive/negative branch)
%
%---------------------------------------------------------------------------------------------------- 
%   @version    $3.0$
%   @date       $6-jul-2021$ $25-sep-2020$ $09-may-2020$ $28-apr-2020 $  
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%---------------------------------------------------------------------------
function    out     =   genBOKchirp_MultiUser(set,flg)
%----------------------------- PARAMETER INITIALIZATION ------------------%
%
    B       =   set.sig.B;                                                  %   Signal Bandwidth (Hz)
    Tc      =   set.sig.Tc;                                                 %   Chirp period (sec.)
    N       =   set.samp.Ns;                                                %   Number of chirp samples    
    M       =   set.M;                                                      %   Number of satellites
    Nnc     =   set.samp.Nnc;                                               %   Number of non-coherent integrations
    idsat   =   set.idsat;
    %
    Nm      =   length(M);                                                  %   Number of different "number of satellites" to be simulated 
    %-  Get Signal parameters: initial freq. and chirp rate
    %
    if Nm == 1
        mu      =   set.sig.mu;   
        f0      =   set.sig.f0;
        %
        tau     =   set.tau;
        fD      =   set.fD;
    end
    %    
    %-  If a given satellite is selected, initialize its particular
    %   parameters. 
    if( ~isempty(idsat) )
        M       =   set.Mvis;
        mu      =   mu(idsat,:);
        f0      =   f0(idsat);
        tau     =   tau(idsat);
        fD      =   fD(idsat);
    end
    %   
%-------------------------------- GENERATE BOK-CHIRP ---------------------%
%
    for jj = 1:Nm   % Generation for each constellation size
        %
        if set.flg 
            disp(['> Generating ' num2str(M(jj)) ' users']);
            disp('% ...');
        end
        %-  Initialize output variables
        %
        Ck                  =   zeros(M(jj),N);
        if( flg ~= 2 )
            ck_dc               =   zeros(M(jj),N);
            ck_dcn              =   zeros(M(jj),N);
            r_dc                =   zeros(M(jj),N);
            r_dcn               =   zeros(M(jj),N);  
        else
            ck_dc1               =   zeros(M(jj),N);
            ck_dcn1              =   zeros(M(jj),N);
            ck_dc2               =   zeros(M(jj),N);
            ck_dcn2              =   zeros(M(jj),N);
            %
            r_dc1                =   zeros(M(jj),N);
            r_dcn1               =   zeros(M(jj),N);  
            r_dc2               =   zeros(M(jj),N);
            r_dcn2               =   zeros(M(jj),N);
        end
        %
        %-  Get Signal parameters: initial freq. and chirp rate
        %
        if( Nm ~= 1 )
            [mu,f0]         =   initSigParams(B,Tc,M(jj),flg);
            tau             =   set.tau(1:M(jj));
            fD              =   set.fD(1:M(jj));
        end
        %
        for ii = 1:M(jj)    % Generation for each satellite in constellation
            %-  Get BOK-chirp for i-th satellite in a constellation with
            %   M(jj) satellites
            %
            outt                =   genBOKchirp(f0(ii),mu(ii,:),B,tau(ii),...
                                                fD(ii),Tc,Nnc);
            %
            %-  Get de-chirp results
            %
            Ck(ii,:)            =   outt.Ck;
            if( flg ~= 2 )
                ck_dc(ii,:)     =   outt.ck_dc;
                ck_dcn(ii,:)    =   outt.ck_dcn;
            else
                ck_dc1(ii,:)    =   outt.ck_dc1;
                ck_dcn1(ii,:)   =   outt.ck_dcn1;
                %
                ck_dc2(ii,:)    =   outt.ck_dc2;
                ck_dcn2(ii,:)   =   outt.ck_dcn2;
            end
            %
            %-  Save out structure
            %
            saveOutBOKchirp_MultiUser;
            %            
        end
        %
        %-  Save de-chirp results
        %
        if( flg ~= 0 )
            Cr                          =   sum(Ck,1);
            for ii = 1:M(jj)
                if( flg ~= 2 )
                    r_dc(ii,:)          =   Cr.*ck_dc(ii,:);
                    r_dcn(ii,:)         =   Cr.*ck_dcn(ii,:);
                    %
                else
                    r_dc1(ii,:)         =   Cr.*ck_dc1(ii,:);
                    r_dcn1(ii,:)        =   Cr.*ck_dcn1(ii,:);
                    %
                    r_dc2(ii,:)         =   Cr.*ck_dc2(ii,:);
                    r_dcn2(ii,:)        =   Cr.*ck_dcn2(ii,:);
                    %                    
                end
            end            
            %
            out.BOK.MU(jj).f0       =   f0;
            out.BOK.MU(jj).mu       =   mu;
            %
            out.BOK.MU(jj).Cr       =   Cr;
            if( flg ~= 2 )
                out.BOK.MU(jj).r_dc     =   r_dc;
                out.BOK.MU(jj).r_dcn    =   r_dcn;
            else
                out.BOK.MU(jj).r_dc1     =   r_dc1;
                out.BOK.MU(jj).r_dcn1    =   r_dcn1;
                %
                out.BOK.MU(jj).r_dc2     =   r_dc2;
                out.BOK.MU(jj).r_dcn2    =   r_dcn2;
            end
            %
        end
        %
    end
    %
    out     =   out.BOK;    
    %

end