% function [fk,phk,cki]   =   genMultiSlope2(mu,B,tau,fD,Tc,Nnc)
%
%   Description:
%       Generation of the results for the MDS-CSS signal with the given
%       input parameters.
%
%   Inputs:
%       mu:     (2x1) vector with the value of the two chirp rates for the 
%               MDS-CSS signal (Hz^2)
%       B:      Signal Bandwidth (Hz)
%       tau:    Time-Delay (sec.)
%       fD:     Frequency Doppler (Hz)
%       Tc:     Chirp period (sec.)
%       Nnc:    Number of non-coherent integrations 
%
%   Outputs:    MDS-CSS signal results
%
%       fk:     (Nx1) vector with the values corresponding to the MDS-CSS
%               frequency (Hz) at each epoch. A total of N = Nnc*ceil(B*Tc)
%               epochs are considered. The frequency values are given in
%               the range of [-B/2,B/2]
%       phk:    (Nx1) vector with the values corresponding to the chirp 
%               phase (rad) at each epoch. NOTE: phk = 2\pi\int fk dk
%       cki:    (Nx1) vector with the values corresponding to the I/Q 
%               samples of the generated chirp signal at each epoch.  
%
%---------------------------------------------------------------------------------------------------- 
%   @version    $2.0$
%   @date       $6-jul-2021$ $23-sep-2020$
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------%
function [fk,phk,cki]   =   genMultiSlope2(mu,B,tau,fD,Tc,Nnc)
%
%------------------------------ INITIALIZE PARAMETERS --------------------%
    %
    A       =   1;                                                          %   Amplitude
    Tsamp   =   1/B;                                                        %   Sampling Time [s/sample]
    N       =   ceil(B*Tc);                                                 %   Number of chirp samples 
    N2      =   floor(N/2);                                                 %   Number of chirp samples in Tc/2
    k       =   (0:(N-1)).';                                                %   Time index [samples]
    k1      =   k(1:N2);                                                    %   Time index for <=Tc/2 [samples]
    k2      =   k(N2+1:end);                                                %   Time index for >Tc/2 [samples]
    %
    tau     =   mod(tau,Tc);                                                %   Time-Delay modulo Tc [sec.]
    Ntau    =   ceil(tau*B);                                                %   Time-Delay modulo Tc [samples]
    %
%--------------------------- CHIRP FREQUENCY GENERATION ------------------%
    %
    fk      =   [];
    for ii = 1:Nnc
        %-  Instantaneous frequency
        %
        fk1         =   mod((mu(1)*(Tsamp*k1)),B);                          %   <= Tc/2 (1st half of the chirp period)
        fk2         =   mod((mu(1)-mu(2))*(Tc/2)+(mu(2)*(Tsamp*k2)),B);     %   > Tc/2 (2ns half of the chirp period)
        %
        %-  Wrap to +/- B/2
        %
        %--     1st half
        %
        idx1        =   fk1 > (B/2);
        idx2        =   fk1 < -(B/2);
        fk1(idx1)   =   fk1(idx1) - B;
        fk1(idx2)    =   B + fk1(idx2);
        %
        %--     2nd half
        %
        idx1        =   fk2 > (B/2);
        idx2        =   fk2 < -(B/2);
        fk2(idx1)   =   fk2(idx1) - B;
        fk2(idx2)    =   B + fk2(idx2);
        %
        %
        %-  Include Doppler frequency
        %
        fk1          =   fk1 + fD;
        fk2          =   fk2 + fD;
        %
        %- Generate freq.
        %
        fkt     =   [fk1;fk2];
        fk      =   [fk;fkt];
        %
    end
    %-  Control the first part corresponding to t < tau (emulating received
    %   signal)
    %
    if Ntau ~= 0
        fk      =   [fk(end-Ntau+1:end);fk(1:end-Ntau)];    
    end
    %
%----------------------------- CHIRP PHASE GENERATION --------------------%
    %    
    Dk      =   Tsamp;                    
    phk     =   2*pi*cumsum(fk)*Dk;
    %
%---------------------------- CHIRP SIGNAL GENERATION --------------------%
    %
    cki     =   A*exp(1j*phk);
    %

    
end