% function [fk,phk,cki]   =   genchirp(f0,mu,B,tau,fD,Tc,Nnc)
%
%   Description:
%       Generation of the results for the chirp (CSS) signal with the given
%       input parameters.
%   Inputs:
%       f0:     Initial frequency (Hz)
%       mu:     Chirp rate (Hz^2)
%       B:      Signal Bandwidth (Hz)
%       tau:    Time-Delay (sec.)
%       fD:     Frequency Doppler (Hz)
%       Tc:     Chirp period (sec.)
%       Nnc:    Number of non-coherent integrations 
%
%   Outputs:    CSS signal results
%
%       fk:     (Nx1) vector with the values corresponding to the chirp
%               frequency (Hz) at each epoch. A total of N = Nnc*ceil(B*Tc)
%               epochs are considered. The frequency values are given in
%               the range of [-B/2,B/2]
%       phk:    (Nx1) vector with the values corresponding to the chirp 
%               phase (rad) at each epoch. NOTE: phk = 2\pi\int fk dk
%       cki:    (Nx1) vector with the values corresponding to the I/Q 
%               samples of the generated chirp signal at each epoch.  
%
%---------------------------------------------------------------------------------------------------- 
%   @version    $3.0$
%   @date       $6-jul-2021$ $22-may-2020$ $17-apr-2020$ 
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%--------------------------------------------------------------------------
function [fk,phk,cki]   =   genchirp(f0,mu,B,tau,fD,Tc,Nnc)
%
%------------------------------ INITIALIZE PARAMETERS --------------------%
    %
    A       =   1;                                                          %   Signal amplitude
    Tsamp   =   1/B;                                                        %   Sampling Time [sec./sample]
    N       =   Nnc*ceil(B*Tc);                                             %   Number of chirp samples
    k       =   (0:(N-1)).';                                                %   Time index [samples]
    %
    tau     =   mod(tau,Tc);                                                %   Time-Delay modulo Tc [sec.]
    Ntau    =   ceil(tau*B);                                                %   Time-Delay modulo Tc [samples]
    %
%--------------------------- CHIRP FREQUENCY GENERATION ------------------%
    %    
    tmp         =   f0+(mu*(Tsamp*k-tau));                                  %   Instantaneous frequency
    %-- Get modulo +/- B
    %
    idx         =   tmp < 0;
    tmp(idx)    =   mod(tmp(idx),-B);
    tmp(~idx)   =   mod(tmp(~idx),B);
    fk          =   tmp;
    %
    %-- Wrap to +/- B/2
    %
    idx1        =   fk > (B/2);
    idx2        =   fk < -(B/2);
    fk(idx1)    =   fk(idx1) - B;
    fk(idx2)    =   B + fk(idx2);
    %
    %-- Include Doppler frequency
    %
    fk          =   fk + fD;                                                
    %
    %
%----------------------------- CHIRP PHASE GENERATION --------------------%
    %
    Dk          =   Tsamp;                                                  %   Difference between samples [s]
    phk         =   2*pi*cumsum(fk)*Dk;                                     %   Chirp phase [rad]
    %-  Control the first part corresponding to t < tau (emulating received
    %   signal)
    %
%     if Ntau ~= 0
%         fk      =   [fk(end-Ntau+1:end);fk(1:end-Ntau)];
%         phk     =   [phk(end-Ntau+1:end);phk(1:end-Ntau)];
%     end
    %
    %
%---------------------------- CHIRP SIGNAL GENERATION --------------------%
	%
    cki         =   A*exp(1j*phk);
    %
    
end

