% function    [set,flg]   =   initParams(set,flg)
%
%   Description:
%       General Parameters initialization. It uses the flag and sett 
%       structures generated with the given configuration in setParams.m
%
%   Inputs:
%       set     =   Structure with setting parameters (see setParams.m)
%
%       flg     =   Structure with flag options (see setParams.m)
%
%   Outputs:
%       set     =   Structure with setting parameters, modified including
%                   number of samples and related signal parameters such 
%                   initial CSS frequency and slope. Initialization of
%                   parameter estimation analysis (time-delay and Doppler)
%
%       flg     =   Structure with flag options, modified including the
%                   initialization of flags related with graphics.
%---------------------------------------------------------------------------------------------------- 
%   @version    $4.0$
%   @date       $5-jul-2021$ $3-oct-2020$ $12-jun-2020$ $17-apr-2020$  
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%---------------------------------------------------------------------------
function    [set,flg]   =   initParams(set,flg)    
    %-  Get settings
    %
    B           =   set.sig.B;
    Tc          =   set.sig.Tc;
    %
    Muser       =   flg.Muser;
    graphics    =   flg.graphics;
    ftau        =   flg.ftau;
    fDopp       =   flg.fDopp;
    %
    Nnc         =   set.samp.Nnc;
    %
    %-  Get MAximum tau and Doppler from LEO constelation
    %   LEO low constellation. Code/Doppler compensation with 300 km
    %   considered (if selected)
    %
    if( ftau || fDopp )
        [taumax,fDmax]  =   getLEOparams(set,flg);
    end
    %
    %-
    %
    %-
    %- Init Signal parameters
    Ns      =   Nnc*ceil(Tc*B);                                             %   Number of samples
    n       =   (0:(Ns-1)).';                                               %   Sample index
    sk      =   0;                                                          %   Data bit (set to 0)
    %
    if( Muser == 0 )    %   Get number of satellites
        M   =   1;
    else
        M           =   set.M;
        if( M == 1 )
            flg.Muser = 0;
        end
    end
    Nm          =   length(M);                                              %   Number of different "number of satellites" to be simulated
    %-- Initial CSS freq. (f0) and slope (mu)
    if( Nm == 1 )
        [mu,f0]     =   initSigParams(B,Tc,M,flg.Muser);
    end
    %
    %
    %- Init tau-analysis parameters 
    %
    if ftau
        %
        initTDAnalysisParams();
        %
    else
        if( flg.Muser ~= 0 )
            tau     =   zeros(M(end),1);
        else
            tau     =   0;
        end
    end
    %
    %- Init fD-analysis parameters
    %
    if fDopp        
        %
        initfDAnalysisParams();
        %       
    else
        if( flg.Muser ~= 0 )
            fD     =   zeros(M(end),1);
        else
            fD      =   .1e6;
        end
    end
    % 
    %
    set.tau         =   tau;
    set.fD          =   fD;
    %- Init graphics
    %
    if graphics
        plot            =   flg.plot;
        if( flg.plot.fdchirp )
            if( flg.plot.dchirp.spect == 0 && flg.plot.dchirp.freq == 0)
                flg.plot.fdchirp    =   0;
            end
        end
        %
        if( (ftau + fDopp) == 0 )
            plot.fest       =   0;
        end
        %
        if( ~flg.plot.fdchirp && ~plot.fest && ~plot.frmse ...
                && ~plot.fsens && ~plot.fcomplex)
            flg.graphics    =   0;
        end 
        %
        flg.plot            =   plot;
        %%
    end
    
    %
    %-         Upload set sructure
    %
    set.sig.sk      =   sk;
    set.sig.Tc      =   Tc;
    if( Nm == 1 )
        set.sig.mu      =   mu;
        set.sig.f0      =   f0;
    end
    set.samp.Ns     =   Ns;
    set.n           =   n;    
    set.Fs          =   B;
    %
    set.M           =   M;
    
    %
    %-- tau-analysis
    %
    if flg.ftau
        if( Ntau > 1 )
            set.estimation.tau  =   taue;
        end
        %        
    end
    %
    %-- fD-analysis
    if flg.fDopp
        if( NDopp > 1 )
            set.estimation.fD   =   fDe;
        end
    end
    
end











