%   mainINNUENDO
%
%   Description:
%       Software main file. This file is used to execute the CSS-INNUENDO
%       software tool with the desired configuration parameters set in the
%       setParams.m file. It executes all the selected analyses in the
%       configuration file. The execution of the code depends on the
%       configuration set in the setParams.m file. Please, check this
%       configuration file to control the execution of the code.
%
%   Inputs:
%       Configuration to be set in the setParams.m file
%
%   Outputs:
%
%       sett        =   Structure with setting parameters (see setParams.m)
%           .NB
%
%       flg         =   Structure with flag options (see setParams.m)
%          .graphic
%
%       out         =   Structure with output results (see 
%                   RMSE_Analysis,Complexity_Analysis)
%       out.BOK     =   see genBOKchirp_MultiUser
%       out.sens    =   see Sensitivity_Analysis
%       out.MUI     =   see MUI_Analysis
% out.RMSE
% out.complex
%---------------------------------------------------------------------------------------------------- 
%   @version    $4.0$
%   @date       $8-jul-2021$ $4-jun-2021$ $9-jun-2020$ $17-apr-2020$  
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%---------------------------------------------------------------------------------------------------- 
%
clearvars;
%close all;
clc;
%
%
addpath('genChirp');
addpath('MultiUser');
addpath('ParamEstimation');
addpath('plots');
addpath('SNR_Analysis');
addpath('Sensitivity');
addpath('initParams');
addpath('Complexity');
%
%------------------------------- SETTING PARAMETERS ----------------------%
%
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('%');
disp('Setting parameters for mainINNUENDO.m');
disp('% ...');
%
global analisis;
global B;
global CN0;
[sett,flg]       =   setParams(analisis,B,CN0);
%
%--------------------------- INITIALIZATION PARAMETERS -------------------%
%
%-  Useful flags to ctrl the execution of the code
%
flgt            =   (sett.samp.NB + sett.samp.NTc + sett.samp.Nc) == 3;     
flga            =   flg.fsens || flg.frmse || flg.fcomplex;
%
%-  Initialize parameters
%
if flgt
    [sett,flg]       =   initParams(sett,flg);
end
%
%
%----------------------------- CSS SIGNAL GENERATION ---------------------%
%
%-  Generate signal only if parameter estimation analysis is selected and
%   no other analysis is considered. Only when just 1 signal parameter (B,
%   Tc and CN0) value
%
if( (flg.graphics || flg.Muser) && flgt == 1 && ~flga) 
    %
    disp('Generating BOKchirp signal');
    disp('% ...');
    sett.flg         =   1;
    %
    out.BOK         =   genBOKchirp_MultiUser(sett,flg.Muser); 
    %    
end
%
%

%-      Parameter estimation analysis
% if( (flg.ftau || flg.fDopp) && ~flga )
%     %
%     disp('Performing Parameter Estimation Analysis');
%     disp('% ...');
%     sett.flg         =   0;
%     %
%     out.estimation  =   ParamEstimation_Analysis(sett,flg);
%     %
% end
%

%----------------------------- SENSITIVITY ANALYSIS ----------------------%
%
if flg.fsens
   %
    disp('Performing Sensitivity Analysis');
    disp('%%%%%%%%%%');
    %
    out.sens    =   Sensitivity_Analysis(sett,flg);
    %    
end
%
%----------------------------- MUI ANALYSIS ------------------------------%
%
if flg.fMUI
    disp('Performing Multi-Satellite Analysis');
    disp('%%%%%%%%%%');
    %
    tmp     =   sett;
    %-  Link with Sensitivity analysis
    %
    if flg.fsens           
        %--     Get Tcmin to achieve target sensitivity
        %
        if( sett.sens.opt == 2 )
            tmp.sig.Tc      =   out.sens.Tcopt.Tc;                          %   Set chirp period
            tmp.samp.NTc    =   length(tmp.sig.Tc);                         %   Set number of Tc values to be analyzed
        end
        %
    end
    %            
    out.MUI     =   MUI_Analysis(tmp,flg);
    %    
end
%
%----------------------------- RMSE ANALYSIS -----------------------------%
%
if flg.frmse
    tmp         =   sett;
    %-  Link with sensitivity analysis
    %
    if flg.fsens
        %--     Get Tcmin to achieve target sensitivity
        %
        if( sett.sens.opt == 2 )            
            tmp.sig.Tc      =   out.sens.Tcopt.Tc; 
            tmp.samp.NTc    =   length(tmp.sig.Tc);
            tmp.samp.Nc     =   1;
        else
            flg.fsens = 0;
        end
        %
    end
    %
    %-  Link with MUI analysis
    %
    if flg.fMUI
        tmp.estimation.MUI  =   out.MUI;
    end
    %
    out.RMSE    =   RMSE_Analysis(tmp,flg);        
end
%
%----------------------------- COMPLEXITY ANALYSIS -----------------------%
%
if flg.fcomplex
    disp('Performing Complexity Analysis');
    disp('%%%%%%%%%%');
    %
    out.tmp         =   1;
    out.complex.C   =   Complexity_Analysis(sett,flg.fsens,out);
end
%
%----------------------------- PLOT RESULTS ------------------------------%
%
if flg.graphics
    %
    disp('Ploting results:');
    %
    plotResults(out,sett,flg);
end
%
disp('% Done!!!');
disp('%');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
%
    



