%   setParams
%
%   Description:    
%       Software configuration file. This file is used to
%       configure the CSS-INNUENDO software tools. Signal parameters, 
%       software options (analyses) and configurations to be considered.
%
%   Inputs:
%       Configuration parameters to be set
%
%   Outputs:
%       Simulation tool configuration
%
%       sett    =   Structure with setting parameters
%
%- Signal parameters
%           .M:                 Number of satellites in the constellation
%           .Mvis:              Number of visible satellites
%           .tau:               Simulated time-delay (sec.)
%           .fD:                Simulated frequency Doppler (Hz)
%           .sig.B:             Simulated signal bandwidth (Hz)
%           .sig.Tc:            Simulated chirp periods (sec.)
%
%- SW parameters
%           .samp.Nsb:          Number of samples per bit (To Be Removed)
%           .samp.NB:           Number of simulated bandwidths
%           .samp.NTc:          Number of simulated chirp periods
%           .samp.Nc:           Number of simulated CN0s
%           .samp.Nnc:          Number of non-coherent integrations
%
%- SW Analysis configuration
%-- Param Estimation Analysis
%           .estimation.Ntau:   Number of simulated time-delay (tau) values
%           .estimation.Dtau:   Set frequency resolution (in meters) 
%                               for tau estimation     
%           .estimation.NDopp:  Number of simulated freq. Doppler values
%           .estimation.Df:     Set frequency resolution (in meters) 
%                               for Doppler estimation 
%
%-- Sensitivity Analysis (if fsens = 1)
%           .sens.Rs:           Number of statistical realizations for the 
%                               estimation of the probability of false
%                               alarm (Pfa)
%           .sens.Rd:           Number of statistical realizations for the 
%                               estimation of the probability of detection
%                               (Pd)
%           .sens.opt:          Select optimization option: '0' for Tc_min 
%                               analysis or '1' for CN0_min analysis
%   opt = 1 
%       	.sens.CN0min:       Minimum CN0 to be used for the opt. proc.
%       	.sens.CN0max:       Maximum CN0 to be used for the opt. proc.
%       	.sens.CN00:         Average CN0 to be used for the opt. proc.
%   opt = 2 
%       	.sens.Tcmin:        Minimum chirp period to be used 
%       	.sens.Tcmax:        Maximum chirp period to be used 
%       	.sens.T0:           Average chirp period to be used 
%
%           .sens.flg_nc:       Select non-coherent integration optimization
%           .sens.fit:          Select fiting option to perform statistical 
%                               fitting [1] or not [0]
%           .sens.CN0:          Simulated CN0 for the sensitivity analysis
%           .sens.Pfa:          Target Pfa
%
%-- RMSE Analysis (if frmse = 1)
%           .estimation.fsim:   To perform numerical simulations [1] or not 
%                               (just get CRLB) [0]
%           .estimation.R:      Number of statistical realizations
%           .estimation.CN0:    Simulated CN0 for the RMSE analysis
%           .estimation.const:  Get results for the whole constelation [1] 
%                               or just for one sat. [0]
%           .estimation.zpfact: Zero padding factor  for the fft used to 
%                               estimate parameters
%
%       flg    =   Structure with flag options
%- SW configuration
%-- Signal options
%           .Muser:             Multi-User option for single-user [0],
%                               multi-slope [1], multi-dual slope [2] or
%                               multi-frequency [3]
%           .ftau:              Time-delay (tau) option to carry out the 
%                               analysis [1] or not [0]
%           .fDopp:             Freq. Doppler option to carry out the 
%                               analysis [1] or not [0]
%-- Signal analyses
%           .frmse:             Root Mean Squared Error (RMSE) option to 
%                               carry out the analysis [1] or not [0]
%           .fsens:             Sensitivity option to carry out the 
%                               analysis [1] or not [0]
%           .fMUI:              Multi-User interference option to carry out 
%                               the analysis [1] or not [0]
%           .fcomplex:          Complexity option to carry out the analysis 
%                               [1] or not [0]
%-- SW options
%           .save:              Save option to save results [1] or not [0]
%           .graphics:          Graphic option to plot results [1] or not [0]
%   graphics = 1 (check below, definitions among the code)
%       	.plot.fsig
%       	.plot.fdchirp
%       	.plot.fest
%       	.plot.fmui
%       	.plot.frmse
%       	.plot.fsens
%       	.plot.fcomplex
%
%---------------------------------------------------------------------------------------------------- 
%   @version    $4.0$
%   @date       $5-jul-2021$ $10-jun-2021$ $19-nov-2020$ $17-apr-2020$
%   @author     SPCOMNAV-UAB: Daniel Egea Roca (daniel.egea@uab.es) 
%---------------------------------------------------------------------------
%
function    [set,flg]   =   setParams(analisis,B,CN0)
%------------------------------ SET SIGNAL PARAMETERS --------------------%
%
    %-  Set min and max bandwidth to be simulated (Hz)
    %
    global nsat;
    global nvis;
    global lora;

    if( iscolumn(B))
        B = B(~isnan(B))';
    else
        B = B(~isnan(B));
    end
    if( iscolumn(CN0))
        CN0 = CN0(~isnan(CN0))';
    else
        CN0 = CN0(~isnan(CN0));
    end
    Bmin        =   B;%4.000e6;
    Bmax        =   20e6;
    %
    %-
    %-  Set min and max Chirp period (Tc to be simulated) (sec.)
    %
    Tcmin       =   0.1*1e-3;
    Tcmax       =   22.5e-3;
    %
    %-
    %-  Set min and max carrier-to-noise ratio (CN0 to be simulated)
    %   (dB-Hz)
    %
    CN0min      =   CN0;%linspace(35,50,16);
    CN0max      =   50;
    %
    Muser       =   2;                                                      %   Multi-User option for single-user '0', multi-slope '1'
                                                                            %   multi-dualslope '2' or multi-freq '3'
    %-  Set multi-user parameters (Number of users)
    %
    idsat   =   [];
    if( Muser ~= 0 )        
        Nm      =   1;                                                      %   Number of diferent "number of users" to be simulated
        Mmax    =   nsat;                                               %   Maximum number of users to be simulated                                                               
        Mvis    =   nvis;%[5 20 30];                                                 %   Number of visible satellites
        
    end
%
%-------------------------------- SET SW PARAMETERS ----------------------%
%
    set.str     =   'B10_20MHz_Tc10ms_M50_200_Mv5_20';         %str to be appended in output file saved in server
    %
    NB          =   length(B);                                                     %   Number of simulated bandwidths
    NTc         =   1;                                                     %   Number of simulated chirp periods
    Nc          =   length(CN0);                                                     %   Number of simulated CN0s
    Nnc         =   1;%[2 3 5 6];                                          %   Number of non-coherent integrations
    Nsb         =   1;                                                     %   Number of samples per bit
    %
%
%------------------------------- SET SW CONFIGURATION --------------------%
    % Signal options
    %    
    ftau        =   1;                                                      %   Time-delay (tau) option to carry out the analysis [1] or not [0]
    fDopp       =   0;                                                      %   Freq. Doppler option to carry out the analysis [1] or not [0]
    %
    % Signal analyses
    %
    
    fsens       =   1;                                                      %   Sensitivity option to carry out the analysis [1] or not [0]
    frmse       =   0;
    fcomplex    =   0;
    fMUI        =   0;
    if(analisis(2) == 1)
        fMUI        =   1;                                                    %   Multi-User interference option to carry out the analysis [1] or not [0]
    end
    if(analisis(3) == 1)
            frmse       =   1;                                                      %   Root Mean Squared Error (RMSE) option to carry out the analysis [1] or not [0]
            fMUI        =   1;       % Necesario para obtener precisión
    end
    if (analisis(4) == 1)
            fcomplex    =   1;                                                      %   Complexity option to carry out the analysis [1] or not [0]
    end

            
            %
    % SW options
    %
    save        =   1;                                                      %   Save option to save results [1] or not [0]
    
    %
    %- Error control
    %
    if( Muser > 3 )
        error('Muser should be a value between 0 - 3. Please, check the value of Muser in setParams.m');
    end
    %
    %-
    flg.comp    =   0;                                                      %   Use Code/Doppler Compensation
    %
    %-  Set graphic options                       
    %    
    graphics    =   1;                                                      %   Graphic option to plot results [1] or not [0]
    if graphics
        %
        plot.fsig               =  0;                                      %   Plot signal generation results
        if plot.fsig
            plot.sig.waveform   =   1;                                      %   Plot signal waveform
            plot.sig.freq       =   1;                                      %   Plot signal frequency
        end
        %
        plot.fdchirp            =   0;                                      %   Plot dchirp process results
        if plot.fdchirp
            plot.dchirp.spect   =   1;                                      %   Plot dechirp spectrum
%             plot.dchirp.freq    =   1;                                      %   Plot dechirp freqs.
        end
        %
        plot.fest               =   0;                                      %   Plot ParameterEStimation results
        if plot.fest
            % To include more options
        end
        %
        if (analisis(2) == 1)
            plot.fmui               =   1;                                      %   Plot MUI analysis results
            if plot.fmui
                plot.mui.users      =   0;
            end
        else
            plot.fmui               =   0;
        end
        %
        if (analisis(3) == 1)
            plot.frmse              =   1;                                      %   Plot RMSE Analysis results
            if plot.frmse
                % To include more options
            end
        else
            plot.frmse               =   0;
        end
        %
        if (analisis(1) == 1)
            plot.fsens              =   1;                                      % Plot sensitivity analysis results
            if plot.fsens
                plot.sens.prob      =   1;                                      % Plot probabilities of false alarm and detection results
                plot.sens.ROC       =   1;                                      % Plot ROCs (Pd_vs_Pfa)
                plot.sens.params    =   1;                                      % Plot statistical distribution fiting results
                plot.sens.fit       =   1;
            end
        else
            plot.fsens              =   0;
        end
        %
        if (analisis(4) == 1)
            plot.fcomplex           =   1;                                      % Plot complexity analysis results        
        else
            plot.fcomplex           =   0;
        end
        %
%         if( Muser ~= 0 )
%             plot.MU             =   0;
%             if plot.MU 
% 
%             end
%         else
%             plot.MU             =   0;
%         end
        %
    end
%
%-------------------------- SET SW ANALYSIS CONFIGURATION ----------------%
%    
    %-  Set tau analysis parameters
    %
    if ftau        
        Ntau    =   1;                                                      %   Number of simulated tau's for the estimation analysis of a single satellite
        tau     =   0;                                                      %   Set tau value for de case when Ntau = 1
        Dtau    =   5;                                                      %   Set "precission" (in meters) for tau estimation (from freq. estimation)         
    end
    %
    %-
    %-  Set Doppler analysis parameters
    %
    if fDopp
        NDopp   =   1;                                                      %   Number of simulated fD's for the estimation analysis of a single satellite
        fD      =   1e2;                                                    %   Set fD value for de case when NDopp = 1
        Df      =   5;                                                      %   Set "precission" (in Hz) for fD estimation (from freq. estimation)         
    end
    %
    %-
    %-  Set RMSE analysis parameters
    %
    if frmse
        fsim    =   0;                                                      %   To perform numerical simulations [1] or not (just get CRLB) [0]
        R       =   1e4;%[5e3 1e4 1e5 1e5];                                 %   Number of statistical realizations
        const   =   0;                                                      %   Get results for the whole constelation '1' or just for one sat. '0'
%         acq     =   0;                                                      %   Consider signal acquisition '1' or not '0'
%         if acq % To Be Removed
%             th  =   [9 9 10 10 10]*1e-5;%[2.5 3 4.5 4]*1e-7;%[2.5 3 5.1 3.65]*1e-7;
%         end
        zpfact  =   1;%[1 1 2 2];                                           %   ZP factor  for the fft used to estimate parameters
    end
    %
    %-  Set Sensitivity analysis parameters
    %
    if fsens
        %-  Statistical realizations
        %
        Rs      =   1e3;                                                    %   Number of statistical realizations for the false alarm probability estimation    
        Rsd     =   1e3;                                                    %   Number of statistical realizations for the probability of detection estimation 
        %
        opt         =   2;                                                  %   Select optimization option: '0' for Tc_min analysis or '1' for CN0_min analysis
        flg_nc      =   0;                                                  %   Select non-coherent integration optimization
        fit         =   1;                                                  %   Select fiting option to perform statistical fitting [1] or not [0]
        %
        Pfa         =   1e-5;                                               %   Target Pfa
        if( opt == 1 )
            CN0min  =   35;                                                 % Minimum CN0 to be used for the optimization process
            CN0max  =   60;                                                 % Maximum CN0 to be used for the optimization process            
            CN00    =   45;                                                 % Average CN0 to be used for the optimization process
        elseif( opt == 2 )
            Tmin    =   0.11e-3;                                             % Minimum chirp period to be used for the optimization process
            Tmax    =   5e-3;%[25 18 8 8]*1e-3;                            % Maximum chirp period to be used for the optimization process
            T0      =   5e-3;%[20 15 8 8]*1e-3;                         % Average chirp period to be used for the optimization process
        end
    end
    %
    %-
    %-  Complexity analysis parameters
    %
    if fcomplex
        % To include more options
    end
    %
    %-
%
%--------------------------- DEFINE SET & FLG STRUCTURES -----------------%
%
    initsetAndflg;   
%
 
end






